      PROGRAM thomas
****************************************************************
*
*  Version: 03/20/02
*
*  Temperature-dependent Thomas-Fermi average-atom model
*
*  INPUT to program:    described in header of subroutine readat
*  OUTPUT from program: described in header of subroutine selfc
* 
*  SUBROUTINES called:  
*                 readat---- 
*                        purpose:  read in data from unit(5)
*                          input:  from unit 5 only
*                         output: 
*                             jz  = nuclear charge Z
*                             aw  = atomic weight (gm/mol)
*                          rplas  = Wigner-Seitz radius (a0)
*                           amlo  = lower bound on mu (a.u.)
*                           amhi  = upper bound on mu (a.u.)
*                             T1  = initial temperature (eV)
*                             T2  = final temperature (eV)
*                             dT  = delta T (eV)
*
*                setgrd----   
*                        purpose:  setup radial grid and initial potential
*                          input: (from readat) 
*                             jz  = nuclear charge Z 
*                             aw  = atomic weight (gm/mol)
*                           rplas = Wigner-Seitz radius (a0)
*                         output: 
*                           mcav  = index of rplas on radial grid
*                             (through common)
*                           radial grid and z(r) initial charge distribution
*
*                 selfc----
*                          input: (from readat) 
*                             jz  = nuclear charge Z
*                           amlo  = lower bound on mu (a.u.)
*                           amhi  = upper bound on mu (a.u.)
*                             T1  = initial temperature (eV)
*                             T2  = final temperature (eV)
*                             dT  = delta T (eV)
*                          input: (from setgrd) 
*                            mcav = index of rplas on radial grid
*                         output: to unit(3) 'thomas.dat' 
*                                 to unit(1) 'rho.dat'                                     
* 
*               
******************************************************************************               
      implicit doubleprecision(a-h,o-z)
      t0 = mclock()

***  initialize gausian coordinates for routine fdinc(aj,b,amu)

      call inint

***  read data for this run from unit 5

      call readat(jz,aw,rplas,amlo,amhi,T1,T2,dT)

***  set p radial grid and starting potential

      call setgrd(jz,aw,rplas,mcav,*911)

***  carry out self-consistent solution to TF equation.
      call selfc(jz,mcav,amlo,amhi,T1,T2,dT,*911)

      ttot = mclock()
      dt = (ttot-t0)/100d0
      write(6,1000) dt
 1000 format(/'  time =',f10.2,' sec')
      stop
 911  stop ' error exit'
      end

      subroutine readat(jz,aw,rplas,amlo,amhi,T1,T2,dT)
**********************************************************************
*
*  Read in data and calculate plasma radius
*
*  input card #1  ident (a4) atomic symbol
*  input card #2  jz = nuclear charge
*                 aw = atomic wgt 
*                 den= density in gm/cc
*  input card #3  bounds on mu in (a.u.)  amlow < mu < amhi
*  input card #4   T(initial), T(final), delta T (all in eV)
* 
*
*    OUTPUT:  jz = Z (nuclear charge)
*             aw = A (atomic weight)
*           rplas= R (Wigner-Seitz plasma radius)
*           amlo =  lower bound on mu (a.u.)
*           amhi =  upper bound on mu  (a.u.)        
*           T1   =  initial temperature (eV)
*           T2   =  final temperature (eV)
*           dT   =  delta T (eV)
*
*    INPUT: only from unit 5 as given below
*
*********************************************************************
      implicit doubleprecision(a-h,o-z)
      character*4 ident
      character*8 version
      common/phycon/alpha,pi,ev,bohr,ryd
      data avag/6.022d-01/
      data version/'03/20/02'/

***  read ident (a4)

      read(5,1000)  ident
 1000 format(a)

***  read Z, A, density

      read(5,*)  jz,aw,den

*** Vol/mol = A/rho (cc/mol)

      vmol = aw/den

***  Vol/atom = Vol/mol / Atom/mol = vmol/avag (Angstrom^3/atom)
***  (A^3/cc = 10^24)

      vatom = vmol/avag

***  vatom = (4*pi/3)*Ratom^3

      ratom = (vatom*3d0/pi/4d0)**(1d0/3d0)

***  convert to a0 units
 
      rplas = ratom/bohr       
        
      write(6,1010) ident,version,jz,aw,den,rplas

 1010 format(8x,' Thomas-Fermi for ',a,6x,'Version:',a//
     1       '               Z =',i4  ,'         A=',f7.2/
     2       '         density =',f6.2,'  rplas =',f7.4)

***  read bounds for mu in (a.u.)
      read(5,*) amlo, amhi 

      write(6,1030) amlo,amhi
 1030 format('   Bounds: ',f10.1,' <  mu < ',f10.1 ) 

***  read  temperatures: Tini, Tfin, delta T (all in eV)

      read(5,*) T1,T2,DT

      write(6,1040) T1,T2,DT
 1040 format('  T1 =',f12.6,'  T2 =',f12.6,'  DT =',f12.6/)
   
      return
      end
 
      block data
***********************************************************************************
*
*    Useful constants:
*
*    alpha = 1/fine structure constant
*       pi = 3.1415...
*     bohr = bohr radius (Angstrom)
*       ev = Ry constant in electeon volts
*      ryd = atomic unit in 1/cm
*
*********************************************************************************
      implicit doubleprecision(a-h,o-z)
      common/phycon/alpha,pi,ev,bohr,ryd
      data alpha /   137.035 989 5 d0/ 
      data pi    / 3.141 592 653 589 793 d0/
      data bohr  / 0.529 177 249 d0/
      data ev    /13.605 698 1 d0/
      data ryd   /   219 474.631 42 d0/
      end
      
      subroutine setgrd(jz,aw,rplas,mcav,*)
      implicit doubleprecision(a-h,o-z)
***********************************************************************
*
*  routine to set up grid and nuclear potential
*  revised May 26, 2000 for SCF in plasma
*  version 03/14/02
*
***********************************************************************
      parameter(NGP=500)
      common/radial/r(NGP),rp(NGP),rpor(NGP),h,max
      common/charge/znuc(NGP),z(NGP)
     &      /phycon/alpha,pi,ev,bohr,ryd      
      data nmax /NGP/,hdef/0.03125/,rdef/5e-4/

      r0  = rdef
      h   = hdef
      max = nmax

      write(6,1000) r0,h,max
 1000 format(/' initial grid parameters:'
     &       /' r0=',f13.5,'  h=',f9.5,'   max=',i6)

**********    set up grid

      r(1)=0.0
      rp(1)=r0
      rpor(1)=0.0
      DO i=2,max
           rp(i)=dexp((i-1)*h)
           r(i)=r0*(rp(i)-1d0)
           rp(i)=r0*rp(i)
           rpor(i)=rp(i)/r(i)
      END DO

***  determine nearest point to plasma radius

      DO i = 1,max
        IF(r(i).lt.rplas) THEN
          mless = i
        END IF
      END DO
      mgrt = mless + 1
      dless = rplas - r(mless)
      dgrt = r(mgrt) - rplas
      IF(dgrt.lt.dless) THEN
        rnew = r(mgrt)
        mcav = mgrt
      ELSE
        rnew = r(mless)
        mcav = mless
      END IF

***  readjust r0 so that rplas = r(mcav)
      r00 = rplas/(dexp(h*(mcav-1))-1d0) 

***  find new grid

      write(6,1010) r0,r00,mcav
 1010 format(' modified r0:'
     &       /' orig=',1p,e15.5,'  new=',e15.5,'  mcav=',i6/)
      
      rp(1)=r00
      DO i=2,max
         rp(i)=dexp((i-1)*h)
         r(i)=r00*(rp(i)-1d0)
         rp(i)=r00*rp(i)
         rpor(i)=rp(i)/r(i)
      END DO

***  verify that rplas = r(mcav)
      write(6,1020) mcav,r(mcav),rplas
 1020 format('  r(',i3,') =',f9.4/
     1       '  Rplas  =',f9.4/)


***  calculate nuclear rms radius (Johnson & Soff) ADNDT 33, 405 (1985)

      rnuc = 0.836d0 * aw**(1d0/3d0) + 0.570d0 
      cnuc =sqrt(5d0/3d0)*rnuc
      c = 1d-5*cnuc/bohr 

***  fill in nuclear potential and Coulomb potential

      DO i=1,max
        IF(r(i).lt.c) THEN
          z(i)=jz*r(i)*(1.5-0.5*(r(i)/c)**2)/c
          inuc=i
        ELSE
          z(i)=jz
        END IF
      END DO      

      tnuc = 0
      write(6,1030) rnuc,cnuc,tnuc,inuc
 1030 format('  r(rms) =',f6.4,'  c =',f6.4,'   t =',f4.2/
     &       '  i(nuc) =',i5/)

****  add a square box potential to neutralize things outside cavity

      DO i = 1,mcav 
         znuc(i) = z(i) 
         z(i) = z(i) - jz*(1.5d0 - 0.5d0*(r(i)/rplas)**2)*r(i)/rplas          
      END DO
      DO i = mcav+1,max
         znuc(i) = z(i)
         z(i) = 0d0
      END DO

***  initialize arrays for subroutine yfun
 
      call inidat

      return

 901  return 1
      end

      subroutine selfc(jz,mcav,amlo,amhi,T1,T2,dT,*)
***************************************************************************
*
*    Self-consistent solution to TF equation
*
*    INPUT :  jz = Z (nuclear charge)
*           mcav = index if rplas on radial grid [rplas = r(mcav)]
*           amlo =  lower bound on mu (a.u.)
*           amhi =  upper bound on mu  (a.u.)        
*           T1   =  initial temperature (eV)
*           T2   =  final temperature (eV)
*           dT   =  delta T (eV)
*
*   OUTPUT:  file 'thomas.dat' connected here as unit(3)
*           for T = T1..T2 in step dT
*     the file contains in each record: (13f12.4)
*           TkeV = temperature  (keV)
*           aAng = R (Angstrom)
*         amukev = mu (keV)
*          PMbar = P (Mbar)
*          fpotn = E(pot)/ZkT
*          fkinn = E(kin)/ZkT
*          fpvn  = PV/ZkT 
*            sok = S/k
*            tm1 = 5/2 PV (keV)
*            tm2 = 1/6 E(e-nuc) (keV)
*            tm3 = 7/6 E(e-e) (keV)
*            tm4 = - Z mu (keV)
*          tskev = TS (keV)
*
*            file 'rho.dat'  connected here as unit(1)
*        this is a summary file prepared to give the density as a function
*        of r suitable for graphic display:
*        for each i from 1 to mcv, we give in each record (1p,4e16.6)
*           r(i)    =  ith radial grid point
*           rhob(i) =  bound component of radial density                   
*           rhoc(i) =  continuum component of radial density
*           rho(i)  =  radial density
*
*********************************************************************
      parameter(NGP=500)
      implicit doubleprecision(a-h,o-z)
      
      common/radial/r(NGP),rp(NGP),rpor(NGP),h,max
     &      /phycon/alpha,pi,ev,bohr,ryd
     &      /charge/znuc(NGP),z(NGP)
     &      /density/rho(NGP)
      common/argment/a2,a3,a4,i5
      dimension u(NGP),v(NGP),y(NGP),rhoc(NGP)

      external fnorm
      data  NTMX /200/, epps /1d-9/ 
      data relerr /1d-12/, abserr /1d-12/, ferr /1d-14/
      data a2kv /0.027211d0/, a2Mbar /294.2101d0/
     
      q = jz  
  
      DO i = 1,max
        u(i) = 0d0
        v(i) = 0d0
        rho(i) = 0d0
      END DO

****  convert temperatures to a.u.

      T1au =  T1/(2*ev)
      T2au =  T2/(2*ev)
      dTau =  dT/(2*ev)

      IF(T2au.eq.0d0.or.DTau.eq.0d0) THEN
          NTMP = 1
      ELSE
          NTMP = 1 + nint((T2au-T1au)/DTau)
      END IF

      open(unit=3,file='thomas.dat',form='formatted',status='unknown',
     1      position='append')

      DO NT = 1,NTMP

         Temp = T1au + dTau * (NT-1)
 
         write(6,1000) 2 * ev * Temp
 1000    format('    kT =',f12.5,'  eV')
      
***  rho ==> 4 \pi r^2 rho    of notes      

          frfac = 2d0*sqrt((2d0*Temp)**3)/pi

***  this starts an iteration loop over the effective potential

          delm = 1.0d0
          aold = 0d0

****  set up the four arguments of fnorm(t,q,Temp,frfac,mcav)
          a2 = q
          a3 = Temp
          a4 = frfac
          i5 = mcav

          DO ipot = 1,NTMX

            IF(ipot.lt.4) THEN
                dell = 1d-3
                ain1 = amlo
                ain2 = amhi
             ELSE IF(delm.lt.0.2d0) THEN
                ddd = 100*dmax1(epps,delm)
                ain1 = amu + ddd
                ain2 = amu - ddd
                dell = 1d-9
             ELSE 
                ain1 = amu + 10*delm
                ain2 = amu - 10*delm
                dell = 1d-6
             END IF

*** solve    Z = int 4 pi r^2 rho(mu) for mu

             t = hybrid(fnorm,ain1,ain2,dell)
             
             amu = t
 
             itot = ipot

***  test convergence

            delm = abs(1d0-aold/amu)

            aold = amu

***  escape when delm < epss
            
            if(delm.lt.epps) go to 910

***  calculate the Hartree screening potential for this case

            l = 0
            call yfun(rho,y,l,mcav,*901)

***  use a 50-50 admixture to accelerate convergence

            DO i = 1,mcav
              z(i) = 0.5 * (z(i) + znuc(i) - y(i)*r(i))
            END DO

            DO i = mcav+1,max
               z(i) = 0d0
            END DO

****  end ipot loop

         END DO

***   end of loop for a single temperature

 910  continue

***   summarize iteration solution

       write(6,1010) itot
 1010  format('  Converged after ',i3,' loops') 

***  calculate the continuum (E>0) contribution to the density.

        cau = amu/Temp
        ORD = 0.5d0       
        u(1) = 0d0
        DO i = 2,mcav
          b = z(i)/(r(i)*Temp)
          x = cau + b
          if(b.lt.0.and.i.eq.mcav) then
            b = 0.0
          end if
          rhoc(i) = fdinc(ORD, B, X)           
          rhoc(i) = frfac * rhoc(i) * r(i)**2
          u(i) = rhoc(i) * rp(i)
          v(i) = (rho(i)-rhoc(i)) * rp(i)
        END DO
        encon = rint(u,1,mcav,7,h)
        enbnd = rint(v,1,mcav,7,h)
        Zi = rhoc(mcav)*r(mcav)/3

        write(6,1020) enbnd,encon,Zi
 1020   format('  Nbound =',f10.6,5x,'Ncont =',f12.6,
     2          '  Zion  =',f12.6)

****  end of loop evaluate pressure (au)

      xp = amu/Temp

      pfac = sqrt((2d0*Temp)**5)/(6*pi**2)

      ORD3 = 1.5d0           
      fdd =  fd(ORD3,xp)
 
      Press = pfac *  fdd

      write(6,1030) Temp,amu,Press
 1030 format(4x,'kT =',f12.6,8x,'mu =',f12.6,6x,'P =',f12.6,' a.u.'/) 

***  convert to practical units and write on unit 3     
      
      TkeV = Temp*a2kv
      amukev = amu*a2kv
      PMbar = a2Mbar * Press

***  entropy and internal energy.

***  calculate Een and Eee

      DO i = 1,mcav
        u(i) = - znuc(i) * rho(i) * rpor(i)
        v(i) = y(i) * rho(i) * rp(i)
      END DO
      Een = rint(u,1,mcav,7,h)
      Eee = 0.5d0 * rint(v,1,mcav,7,h)

      vol = 4 * pi * r(mcav)**3 / 3d0
      
      entr = 2.5d0 * Press * vol + (Een+7*Eee)/6d0 - amu * jz

      ekin = 0.5 * (3 * Press * vol - Een - Eee)
      epot = Een + Eee

      ekinn = a2kv * ekin
      epotn = a2kv * epot
      epvn  = a2kv * Press * vol
      anmu = a2kv * jz * amu
      etotkv = a2kv * (ekin+epot)
      tskev = a2kv * entr
      tm1 = a2kv * 2.5d0 * Press * vol
      tm2 = a2kv * Een / 6d0
      tm3 = a2kv * Eee * 7d0/6d0
      tm4 = -anmu

      aAng = bohr * r(mcav)
      fkinn = ekin/(jz*Temp)
      fpotn = epot/(jz*Temp)
      fpvn  = Press * vol /(jz*Temp)
      sok = entr/Temp

      write(3,2000) TkeV,aAng,amukev,PMbar,fpotn,fkinn,fpvn,sok,
     1              tm1,tm2,tm3,tm4,tskev
 
 2000 format(13f12.4)

      END DO

      close(unit=3)
       
**** write output to a file for graphics in the final case
       
      open(unit=1,file='rho.dat',form='formatted',status='unknown')

      DO i = 1,mcav
       write(1,3000) r(i),rho(i)-rhoc(i),rhoc(i),rho(i)
 3000  format(1p,4e16.6)
      END DO 
 
      return
 901  stop
      end      
      
      doubleprecision function fnorm(amu)
      implicit doubleprecision(a-h,o-z)
**************************************************************************
*
*  This routine evaluates the function
*
*                   fnorm(mu) = Norm(mu) - Z
*  where
*                   Norm = Int_0^\R 4\pi r^2 rho(i)
*  with
*                   rho(i) = frfac * I_1/2[(mu-V(r))/kT]
*
*  the parameters (Z,T,frfac, and mcav) are transferred from routine selfc
*  through the common block common/argment/q,Temp,frfac,mcav

*
**************************************************************************
      parameter(NGP=500,RELERR=1d-14)
      common/radial/r(NGP),rp(NGP),rpor(NGP),h,max
     &      /charge/znuc(NGP),z(NGP)
     &      /density/rho(NGP)
      common/argment/q,Temp,frfac,mcav
      dimension u(NGP)
 
 
***  fill in the radial density rho(r(i))
***  fill in rho * rp(i) for integration
        ord = 0.5d0  
        u(1) = 0d0
        DO i = 2,mcav
          x = (amu+z(i)/r(i))/Temp
          IF(x.gt.10000d0) THEN
            rho(i) = 2d0*dsqrt(x**3)/3d0
          ELSE       
            rho(i) = fd(ord,x)
          END IF
          rho(i) = frfac * rho(i) * r(i)**2
          u(i) = rho(i) * rp(i)
        END DO

***    here is the integral of the density      
        Zf = rint(u,1,mcav,7,h)

***     here is the difference between integral of density and Z
***     we are seeking a zero of this function

        fnorm = Zf - q

        return
        end     

      doubleprecision recursive function hybrid(f,x1,x2,delx) result (b)
        ! find a root of f(x)=0 in the interval [x1,x2]
        ! return estimate of the root with absolute accuracy delx
        ! x1 and x2 must bracket the root, i.e. f(x1)*f(x2) < 0.
        ! (this does not cover roots of even order!)
        ! adapted from netlib fmm/zeroin.f 
        ! see http://www.netlib.org/faq.html
        implicit none
        doubleprecision, external   :: f       ! function f(x)
        doubleprecision, intent(in) :: x1, x2  ! initial bracket of root
        doubleprecision, intent(in) :: delx    ! desired absolute accuracy 

        integer, parameter :: jmax=20
        integer j
        doubleprecision a,c,d, fa,fb,fc,fd, t, dm,ds,dd, df

        ! basic idea:
        ! a, b are the points from which the next secant approximation
        !      will be computed
        ! b, c form a proper backet for the root
        !      loop invariant: abs(f(b)) <= abs(f(c))
        ! if the secant method produces an undesirable approximation,
        ! we take an bisection step instead
        ! -> combines speed of secant method with stability of bisection

        ! initialize
        a=x1
        b=x2
        fa=f(a)
        fb=f(b)
        if (fa*fb.gt.0.d0) then
          write(6,*)  ' hybrid: root must be bracketed'
          write(6,*)  ' x1=',x1,'  x2=',x2,'  delx =',delx
          stop         
        end if
        if (fa.eq.0.d0) then
          b=a
          return
        endif
        if (fb.eq.0.d0) return
        c=a
        fc=fa

        do j=1,jmax
          ! loop invariants: (i)   fa*fb*fc != 0
          !                  (ii)  fb*fc < 0
          !                  (iii) abs(fb) <= abs(fc)
          if (abs(fc) < abs(fb)) then
            ! interchange b and c
            t=c
            c=b
            b=t
            t =fc
            fc=fb
            fb=t
            ! now a, b are no longer subsequent secant iterations
            ! we therefore restart secant method with the current interval
            a=c
            fa=fc
          endif

          if (abs(b-c) <= delx .or. fb.eq.0.) return
         
          dm=0.5*(c-b)  ! for bisection
          ! since |fb|<|fc| the root will most likely be in [b,b+dm] 
          ! this assumption saves one call to f to actually find out
          ! wether root\in[b,b+dm] or root\in[b+dm,c] !
          df=(fa-fb)    
          if (df == 0.d0) then
            ds=dm               ! can't take secant step
          else
            ds=-fb*(a-b)/df     ! secant step
          endif
          ! secant step is only accepted if it lies within [b,b+dm] 
      if (sign(1.d0,ds) /= sign(1.d0,dm) .or. abs(ds) > abs(dm)) then
            dd=dm               ! reject secant step
          else
            dd=ds               ! accept secant step
            ! if secant method produces only points on one side of root
            ! only one bound of [b,c] approaches the root
            ! thus try to induce a move across the root to make the
            ! other bound also approach the root in order to fulfill 
            ! the convergence criterion
            if (abs(dd) < delx) dd=0.5d0*sign(1.d0,dm)*delx
          endif
          ! new iterate
          d=b+dd
          fd=f(d)
          ! rename variables to fulfill loop invariants (i) and (ii)
          if (fd == 0.d0) then
            b=d
            return
          endif
          ! update secant iterates
          a=b
          b=d 
          fa=fb
          fb=fd
          if (fb*fc > 0.d0) then
            ! assumption that root\in[b,b+dm] was wrong!
            ! replace c by old value of b (now in a)
            c=a
            fc=fa 
          endif
        enddo
        stop 'hybrid: too many iterations'
      end
      
      integer function mclock() 
**************************************************************************
*
*     mclock()  = time in 1/100 sec
*     usage:
*     on entry:      t0 = mclock()
*     before exit:   t1 = mclock()
*     elapsed time = t1-t0 (in sec/100)
*
**************************************************************************
          character*8 date
          character*10 time
          character*5 zone
          integer val(8)

          call DATE_AND_TIME(date,time,zone,val)
          mclock = nint( ( val(8) + 1000 * ( val(7) +
     1       60 * ( val(6) + 60 * val(5) ) ) )/10.)
          
          return
          END

      function rint (f,na,nb,nq,h)
      implicit double precision(a-h,o-z)
c
c  this program calculates the integral of the function f from point na
c  to point nb using a nq points quadrature ( nq is any integer between
c  1 and 14 ).  h is the grid size.
c                                      written by c. c. j. roothaan
c
      dimension c(105),c1(78),c2(27),d(14),f(nb)
      equivalence (c1(1),c(1)),(c2(1),c(79))
      data c1/
     a 1.d0,
     b 2.d0,1.d0,
     c 23.d0,28.d0,9.d0,
     d 25.d0,20.d0,31.d0,8.d0,
     e 1413.d0,1586.d0,1104.d0,1902.d0,475.d0,
     f 1456.d0,1333.d0,1746.d0,944.d0,1982.d0,459.d0,
     g 119585.d0,130936.d0,89437.d0,177984.d0,54851.d0,176648.d0,
     g  36799.d0,
     h 122175.d0,111080.d0,156451.d0,46912.d0,220509.d0,29336.d0,
     h 185153.d0, 35584.d0,
     i 7200319.d0, 7783754.d0,5095890.d0,12489922.d0,-1020160.d0,
     i16263486.d0,  261166.d0,11532470.d0,2082753.d0,
     j 7305728.d0,  6767167.d0, 9516362.d0, 1053138.d0,18554050.d0,
     j-7084288.d0, 20306238.d0,-1471442.d0,11965622.d0, 2034625.d0,
     k  952327935.d0, 1021256716.d0,  636547389.d0,1942518504.d0,
     k-1065220914.d0, 3897945600.d0,-2145575886.d0,3373884696.d0,
     k -454944189.d0, 1637546484.d0,  262747265.d0,
     l  963053825.d0,  896771060.d0, 1299041091.d0, -196805736.d0,
     l 3609224754.d0,-3398609664.d0, 6231334350.d0,-3812282136.d0,
     l 4207237821.d0, -732728564.d0, 1693103359.d0,  257696640.d0/
      data c2 / 5206230892907.d0,5551687979302.d0,3283609164916.d0,
     m 12465244770050.d0,-13155015007785.d0,39022895874876.d0,
     m-41078125154304.d0,53315213499588.d0,-32865015189975.d0,
     m 28323664941310.d0,-5605325192308.d0,  9535909891802.d0,
     m  1382741929621.d0,
     n  5252701747968.d0,  4920175305323.d0,  7268021504806.d0,
     n -3009613761932.d0, 28198302087170.d0,-41474518178601.d0,
     n 76782233435964.d0,-78837462715392.d0, 81634716670404.d0,
     n-48598072507095.d0, 34616887868158.d0, -7321658717812.d0,
     n  9821965479386.d0,  1360737653653.d0/
      data d/2.d0,2.d0,24.d0,24.d0,1440.d0,1440.d0,120960.d0,
     a  120960.d0,7257600.d0,7257600.d0,958003200.d0,958003200.d0,
     b  5230697472000.d0,5230697472000.d0/

      a = 0.0d0
      l = na
      m = nb
      i = nq*(nq+1)/2
      do 100 j = 1,nq
         a = a + c(i)*( f(l) + f(m) )
         l = l + 1
         m = m - 1
         i = i - 1
 100  continue
      a = a/d(nq)
      do 200 n = l,m
        a = a + f(n)
 200  continue
      rint = a*h
      return
      end
      
      subroutine yfun(x,y,l,m,*)
************************************************************************
*
*  this program calculates the Hartree's y-functions
*
*  x  : input function
*  y  : output hartree's y-function : y(l,r)/r =
*            r**(-l-1) * Int {0,r} [ r**l x(r) dr ] 
*               + r**l * Int{r,inf}[ r**(-l-1) x(r) ] 
*  l  : order of the y function (must be .ge. 0)
*  m  : number of tabulation points for the input function x
*
************************************************************************
      implicit doubleprecision(a-h,o-z)
      
      parameter(NGP=500,LX=15)
      common/radial/r(NGP),rp(NGP),rpor(NGP),h,max
      common/yfunda/xl(NGP,-LX-1:LX)
      dimension x(NGP),y(NGP)
      dimension u(NGP),v(NGP),w(NGP)

      
      v(1)=0.0
      w(1)=0.0
      if(l.lt.0) then
         write(6,*) ' Error in yint: l < 0 '
         return 1
      elseif(l.eq.0) then 
         do 100  i= 2,m
            v(i) = x(i)*rp(i)
            w(i) = x(i)*rpor(i)
 100     continue
         call yint(v,w,y,u,m,h)
         ym = y(m)
         y(1) = u(1)
         do 120 i = 2,m
            y(i) = y(i)*xl(i,-1) + u(i)
 120     continue 
         m1 = m + 1
         if(m1.le.max) then
           do 140 i = m1,max
              y(i) = ym*xl(i,-1)
 140       continue
         endif
  
         return
      else
         if(l.gt.LX) then
            write(6,1000) l
 1000       format('  Error in yint:  the value of l = ',
     &      i4,' is too large')
            return 1
         endif
         do 180 i = 2,m
            v(i) = x(i)*rp(i)*xl(i,l)
            w(i) = x(i)*rp(i)*xl(i,-l-1)
 180     continue
         call yint(v,w,y,u,m,h)
         ym = y(m)
         do 200 i = 2,m
            y(i) = y(i)*xl(i,-l-1) + u(i)*xl(i,l)
 200     continue
         m1 = m + 1
         if(m1.le.max) then
            do 210  i = m1,max
               y(i) = ym*xl(i,-l-1)
 210        continue
         endif
      endif
      
      return
      end

      subroutine yint (v,w,y,z,m,h)
************************************************************************
*  this program calculates the indefinite integrals y and z using the
*  lagrange integration formula
*
*  y(r) = integral of v from 0 to r
*  z(r) = integral of w from r to infinity
*  m is the maximum tabulation point of v and w (virtual infinity)
*  h is the step size of the radial grid
************************************************************************
      implicit doubleprecision(a-h,o-z)
      logical first 
      parameter(NGP=500)
      dimension v(NGP),w(NGP),y(NGP),z(NGP)
      dimension dy(NGP),dz(NGP)
*
*     lagrange 6 point integration formula
*     ************************************
*      parameter(NO=6,NP=NO/2)
*      dimension aa(NP,NO),a(NO,NP),b(NP)
*      data  da/1440.d0/
*      data  aa/  475.d0,   -27.d0,    11.d0,
*     2          1427.d0,   637.d0,   -93.d0,
*     3          -798.d0,  1022.d0,   802.d0,
*     4           482.d0,  -258.d0,   802.d0,
*     5          -173.d0,    77.d0,   -93.d0,
*     6            27.d0,   -11.d0,    11.d0/
*      data   b/  802.d0,   -93.d0,    11.d0/
************************************************************************
*     lagrange 8 point integration formula
*     ************************************
*      parameter(NO=8,NP=NO/2)
*      dimension aa(NP,NO),a(NO,NP),b(NP)
*      data  da/120960.d0/
*      data  aa/  36799.d0,  -1375.d0,   351.d0,  -191.d0,
*     2          139849.d0,  47799.d0, -4183.d0,  1879.d0,
*     3         -121797.d0, 101349.d0, 57627.d0, -9531.d0,
*     4          123133.d0, -44797.d0, 81693.d0, 68323.d0,
*     5          -88547.d0,  26883.d0,-20227.d0, 68323.d0,
*     6           41499.d0, -11547.d0,  7227.d0, -9531.d0,
*     7          -11351.d0,   2999.d0, -1719.d0,  1879.d0,
*     8            1375.d0,   -351.d0,   191.d0,  -191.d0/   
*      data   b/  68323.d0,  -9531.d0,  1879.d0,  -191.d0/
************************************************************************
*     lagrange 10 point integration formula
*     ************************************
      parameter(NO=10,NP=NO/2)
      dimension aa(NP,NO),a(NO,NP),b(NP)
      data da/7257600.d0/
      data aa/2082753.d0,  -57281.d0,   10625.d0,   -3969.d0,   2497.d0, 
     2        9449717.d0, 2655563.d0, -163531.d0,   50315.d0, -28939.d0,
     3      -11271304.d0, 6872072.d0, 3133688.d0, -342136.d0, 162680.d0,
     4       16002320.d0,-4397584.d0, 5597072.d0, 3609968.d0,-641776.d0,
     5      -17283646.d0, 3973310.d0,-2166334.d0, 4763582.d0,4134338.d0,
     6       13510082.d0,-2848834.d0, 1295810.d0,-1166146.d0,4134338.d0, 
     7       -7394032.d0, 1481072.d0, -617584.d0,  462320.d0,-641776.d0, 
     8        2687864.d0, -520312.d0,  206072.d0, -141304.d0, 162680.d0,
     9        -583435.d0,  110219.d0,  -42187.d0,   27467.d0, -28939.d0,
     a          57281.d0,  -10625.d0,    3969.d0,   -2497.d0,   2497.d0/
      data b/ 4134338.d0, -641776.d0,  162680.d0,  -28939.d0,   2497.d0/
      data first /.true./ 
*
*  note that a different even order method can be used by replacing the
*  dimension and data statements in above block 
************************************************************************

*   initialize the intigration coefficients on first entry

      if(first) then 
         hd = h/da
         do 150 i = 1,NP
            do 100 j = 1,NO
               a(j,i) = aa(i,j)*hd
 100        continue
            b(i) = b(i)*hd   
 150     continue
         first = .false.
      endif

      y(1) = 0.0d0
      z(m) = 0.0d0
      do 200 i = 2,NP
         k = m - i + 1
         y(i) = y(i-1)
         z(k) = z(k+1)
         ii = i - 1
         do 190 j = 1,NO
            y(i) = y(i) + a(j,ii)*v(j)
            z(k) = z(k) + a(j,ii)*w(m-j+1)
 190     continue
 200  continue
      im = NP + 1
      in = m - NP + 1

       do 240 i = im,in
          k = m - i + 1
          dy(i) = 0.0d0
          dz(k) = 0.0d0
          do 230 j = 1,NP
             dy(i) = dy(i) + b(j)*(v(i+j-1) + v(i-j))
             dz(k) = dz(k) + b(j)*(w(k-j+1) + w(k+j))
  230     continue        
  240  continue

      do 250 i = im,in
         k = m - i + 1
         y(i) = y(i-1) + dy(i)
         z(k) = z(k+1) + dz(k)
 250  continue

      in = in + 1
      do 300 i = in,m
         k = m - i + 1
         y(i) = y(i-1)
         z(k) = z(k+1)
         do 280 j = 1,NO
            y(i) = y(i) + a(j,k)*v(m-j+1)
            z(k) = z(k) + a(j,k)*w(j)
 280     continue
 300  continue
      return
      end

      subroutine inidat
***********************************************************
*
*   initialize r^l and r^-l-1  for yfun
*
***********************************************************
      implicit doubleprecision(a-h,o-z)
      parameter(NGP=500,LX=15)
      common/radial/r(NGP),rp(NGP),rpor(NGP),h,max
      common/yfunda/xl(NGP,-LX-1:LX)
      DO  i = 1,max
         xl(i,0) = 1.0d0
      END DO
      DO  ll = 1,LX
         DO  i = 1,max
            xl(i,ll) = xl(i,ll-1)*r(i)
         END DO
      END DO
      DO ll = 1,LX
         xl(1,-ll) = 0.0d0
         DO  i=2,max
            xl(i,-ll) = 1d0/xl(i,ll)
         END DO
      END DO 
      xl(1,-LX-1) = 0.0d0
      DO  i=2,max
         xl(i,-LX-1) = xl(i,-LX)/r(i)
      END DO
       
      return
      end
      
      subroutine inint
      implicit doubleprecision(a-h,o-z)
***************************************************************
*
*  initialize gaussian weights for function fdinc(aj,b,mu)
*
***************************************************************
      parameter(NPT=50)
      common/gauwgt/xm(NPT),wm(NPT)

***  xm(im) are gaussian coordinated -1<xm<1
***  wm(im) are gaussian weights

        call setgau(xm,wm,NPT)
	return

      end

      function fdinc(aj,b,amu)
      implicit doubleprecision(a-h,o-z)
***************************************************************
*
*  evaluate the incomplete fermi-dirac integral fdinc(aj,b,mu)
*
*  for b <|mu| use fd(aj,amu) - Gaussian int from 0..b
*  for b >|mu| Gaussian int from b..b+25
*
*  calls function fd(aj,amu)
*
***************************************************************

      parameter(NPT=50,EUP=25d0)
      common/gauwgt/xm(NPT),wm(NPT)
      
      IF(b.lt.0) THEN
	write(6,*) ' Error in fdinc: b=',b
	stop
      ELSE IF(b.eq.0d0) THEN
	fdinc = fd(aj,amu)
	return
      END IF

      kj = nint(2*aj)+1

      IF(b.lt.abs(amu)) THEN
        fini = fd(aj,amu)
        accum = 0d0
        DO im = 1,NPT
	    y = 0.5d0*(1d0+xm(im))
          arg = y**kj/(1d0+dexp(b*y**2-amu))
          accum = accum + wm(im) * arg
        END DO
        fdinc =  fini -  b * b**aj * accum
      ELSE
        pmax = sqrt(b + EUP)
	blow = dsqrt(b)
	a = 0.5d0*(pmax-blow)
	c = 0.5d0*(pmax+blow)
	accum = 0d0
	DO im=1,NPT
	   p = a*xm(im)+c
	       arg = p**kj/(1d0+dexp(p**2-amu))
	       accum = accum + wm(im) * arg
	END DO 
        fdinc = 2d0 * a * accum
      END IF

      return
      end

      subroutine setgau(x,w,n)
      implicit doubleprecision(a-h,o-z)
      dimension x(n),w(n)
      data eps /1d-15/
      pi = dacos(-1.0d0)
      m = (n+1)/2
      do 120 i = 1,m
         z = cos(pi*(i-0.25d0)/(n+0.5d0))
 100     continue
            p1 = 1.d0
            p2 = 0.d0
            do 110 j = 1,n
               p3 = p2
               p2 = p1
               p1 = (( 2.d0*j-1.d0)*z*p2-(j-1.d0)*p3)/j
 110        continue
            pp = n*(z*p1-p2)/(z*z-1.d0)
            z1 = z
            z = z1-p1/pp
         if(dabs(z-z1).gt.eps) go to 100
         x(i) = -z
         x(n+1-i) = z
         w(i) = 2.d0/((1.d0-z*z)*pp*pp)
         w(n+1-i) = w(i)
 120  continue
      return
      end

       function fd (xnu, alpha) 
       implicit doubleprecision(a-h,o-z)                                         
c w. fullerton, imsl houston, apr 1983 edition.                         
c                                                                       
c fermi-dirac function of order xnu and argument alpha, where           
c xnu is an integer multiple of 0.5 between -0.5 and 4.0                
c inclusive.  this routine is accurate everywhere to approx-            
c imately 10 digits.                                                    
c                                                                       
      dimension fdscs(21,10), fdmcs(33,10), fdbcs(26,10),               
     1  nscs(10), nmcs(10), nbcs(10), alfmax(10)                        
c                                                                       
c  ixer, jxer, kxer flags for error message                             
      data ixer,jxer,kxer/-1,2,2/                                       
c                                                                       
c series for fm5s    on the interval  0.00000e-01 to  2.71828e+00       
c                                        with weighted error   5.75e-11 
c                                         log weighted error  10.24     
c                               significant figures required  10.32     
c                                    decimal places required  10.90     
c                                                                       
      data fdscs(  1, 1) /   2.1468096512  d0 /                           
      data fdscs(  2, 1) /   -.5099961735 5d0 /                         
      data fdscs(  3, 1) /    .1357694225 8d0 /                         
      data fdscs(  4, 1) /   -.0379066840 1d0 /                         
      data fdscs(  5, 1) /    .0108700231 1d0 /                         
      data fdscs(  6, 1) /   -.0031712624 6d0 /                         
      data fdscs(  7, 1) /    .0009364562 1d0 /                         
      data fdscs(  8, 1) /   -.0002790240 1d0 /                         
      data fdscs(  9, 1) /    .0000837159 4d0 /                         
      data fdscs( 10, 1) /   -.0000252565 8d0 /                         
      data fdscs( 11, 1) /    .0000076541 8d0 /                         
      data fdscs( 12, 1) /   -.0000023283 6d0 /                         
      data fdscs( 13, 1) /    .0000007105 2d0 /                         
      data fdscs( 14, 1) /   -.0000002174 1d0 /                         
      data fdscs( 15, 1) /    .0000000666 8d0 /                         
      data fdscs( 16, 1) /   -.0000000204 9d0 /                         
      data fdscs( 17, 1) /    .0000000063 1d0 /                         
      data fdscs( 18, 1) /   -.0000000019 4d0 /                         
      data fdscs( 19, 1) /    .0000000006 0d0 /                         
      data fdscs( 20, 1) /   -.0000000001 8d0 /                         
      data fdscs( 21, 1) /    .0000000000 57d0 /                        
c                                                                       
c series for fm5m    on the interval  1.00000e+00 to  4.00000e+00       
c                                        with weighted error   5.27e-11 
c                                         log weighted error  10.28     
c                               significant figures required  10.55     
c                                    decimal places required  10.94     
c                                                                       
      data fdmcs(  1, 1) /   3.7374179482 1d0 /                         
      data fdmcs(  2, 1) /    .0702840192 74d0 /                        
      data fdmcs(  3, 1) /    .0049804498 35d0 /                        
      data fdmcs(  4, 1) /   -.0102460511 86d0 /                        
      data fdmcs(  5, 1) /    .0046134524 88d0 /                        
      data fdmcs(  6, 1) /   -.0015084055 45d0 /                        
      data fdmcs(  7, 1) /    .0004453628 58d0 /                        
      data fdmcs(  8, 1) /   -.0001323433 48d0 /                        
      data fdmcs(  9, 1) /    .0000410236 76d0 /                        
      data fdmcs( 10, 1) /   -.0000130973 69d0 /                        
      data fdmcs( 11, 1) /    .0000042172 38d0 /                        
      data fdmcs( 12, 1) /   -.0000013563 51d0 /                        
      data fdmcs( 13, 1) /    .0000004356 20d0 /                        
      data fdmcs( 14, 1) /   -.0000001400 58d0 /                        
      data fdmcs( 15, 1) /    .0000000451 47d0 /                        
      data fdmcs( 16, 1) /   -.0000000145 91d0 /                        
      data fdmcs( 17, 1) /    .0000000047 25d0 /                        
      data fdmcs( 18, 1) /   -.0000000015 33d0 /                        
      data fdmcs( 19, 1) /    .0000000004 97d0 /                        
      data fdmcs( 20, 1) /   -.0000000001 61d0 /                        
      data fdmcs( 21, 1) /    .0000000000 527d0 /                       
c                                                                       
c series for fm5b    on the interval  0.00000e-01 to  2.50000e-01       
c                                        with weighted error   2.77e-11 
c                                         log weighted error  10.56     
c                               significant figures required  10.85     
c                                    decimal places required  11.26     
c                                                                       
      data fdbcs(  1, 1) /   3.9538986547 2d0 /                         
      data fdbcs(  2, 1) /   -.0314341746 06d0 /                        
      data fdbcs(  3, 1) /   -.0086537614 36d0 /                        
      data fdbcs(  4, 1) /   -.0000437949 13d0 /                        
      data fdbcs(  5, 1) /    .0003173634 01d0 /                        
      data fdbcs(  6, 1) /    .0000743852 22d0 /                        
      data fdbcs(  7, 1) /   -.0000283890 50d0 /                        
      data fdbcs(  8, 1) /   -.0000089391 79d0 /                        
      data fdbcs(  9, 1) /    .0000045170 94d0 /                        
      data fdbcs( 10, 1) /    .0000007855 81d0 /                        
      data fdbcs( 11, 1) /   -.0000009131 13d0 /                        
      data fdbcs( 12, 1) /    .0000000758 35d0 /                        
      data fdbcs( 13, 1) /    .0000001572 30d0 /                        
      data fdbcs( 14, 1) /   -.0000000671 69d0 /                        
      data fdbcs( 15, 1) /   -.0000000100 31d0 /                        
      data fdbcs( 16, 1) /    .0000000194 28d0 /                        
      data fdbcs( 17, 1) /   -.0000000059 47d0 /                        
      data fdbcs( 18, 1) /   -.0000000020 95d0 /                        
      data fdbcs( 19, 1) /    .0000000025 45d0 /                        
      data fdbcs( 20, 1) /   -.0000000007 47d0 /                        
      data fdbcs( 21, 1) /   -.0000000002 91d0 /                        
      data fdbcs( 22, 1) /    .0000000003 71d0 /                        
      data fdbcs( 23, 1) /   -.0000000001 31d0 /                        
      data fdbcs( 24, 1) /   -.0000000000 290d0 /                       
      data fdbcs( 25, 1) /    .0000000000 573d0 /                       
      data fdbcs( 26, 1) /   -.0000000000 277d0 /                       
c                                                                       
c series for f00s    on the interval  0.00000e-01 to  2.71828e+00       
c                                        with weighted error   3.18e-11 
c                                         log weighted error  10.50     
c                               significant figures required  10.36     
c                                    decimal places required  11.15     
c                                                                       
      data fdscs(  1, 2) /   1.3659874054d0 /                           
      data fdscs(  2, 2) /   -.2438973101 9d0 /                         
      data fdscs(  3, 2) /    .0547680108 5d0 /                         
      data fdscs(  4, 2) /   -.0135159352 3d0 /                         
      data fdscs(  5, 2) /    .0035158670 3d0 /                         
      data fdscs(  6, 2) /   -.0009461111 9d0 /                         
      data fdscs(  7, 2) /    .0002607200 1d0 /                         
      data fdscs(  8, 2) /   -.0000731250 4d0 /                         
      data fdscs(  9, 2) /    .0000207911 1d0 /                         
      data fdscs( 10, 2) /   -.0000059759 5d0 /                         
      data fdscs( 11, 2) /    .0000017329 5d0 /                         
      data fdscs( 12, 2) /   -.0000005062 5d0 /                         
      data fdscs( 13, 2) /    .0000001488 2d0 /                         
      data fdscs( 14, 2) /   -.0000000439 8d0 /                         
      data fdscs( 15, 2) /    .0000000130 5d0 /                         
      data fdscs( 16, 2) /   -.0000000038 9d0 /                         
      data fdscs( 17, 2) /    .0000000011 6d0 /                         
      data fdscs( 18, 2) /   -.0000000003 4d0 /                         
      data fdscs( 19, 2) /    .0000000001 0d0 /                         
      data fdscs( 20, 2) /   -.0000000000 31d0 /                        
c                                                                       
c series for f00m    on the interval  1.00000e+00 to  4.00000e+00       
c                                        with weighted error   6.63e-11 
c                                         log weighted error  10.18     
c                               significant figures required  10.22     
c                                    decimal places required  10.85     
c                                                                       
      data fdmcs(  1, 2) /   2.1728598421d0 /                           
      data fdmcs(  2, 2) /   -.1263072405 5d0 /                         
      data fdmcs(  3, 2) /    .0627040952 7d0 /                         
      data fdmcs(  4, 2) /   -.0248015923 3d0 /                         
      data fdmcs(  5, 2) /    .0086910218 0d0 /                         
      data fdmcs(  6, 2) /   -.0028969250 7d0 /                         
      data fdmcs(  7, 2) /    .0009558659 0d0 /                         
      data fdmcs(  8, 2) /   -.0003167553 1d0 /                         
      data fdmcs(  9, 2) /    .0001054756 2d0 /                         
      data fdmcs( 10, 2) /   -.0000351869 0d0 /                         
      data fdmcs( 11, 2) /    .0000117376 0d0 /                         
      data fdmcs( 12, 2) /   -.0000039134 7d0 /                         
      data fdmcs( 13, 2) /    .0000013044 3d0 /                         
      data fdmcs( 14, 2) /   -.0000004347 7d0 /                         
      data fdmcs( 15, 2) /    .0000001449 1d0 /                         
      data fdmcs( 16, 2) /   -.0000000483 0d0 /                         
      data fdmcs( 17, 2) /    .0000000161 0d0 /                         
      data fdmcs( 18, 2) /   -.0000000053 6d0 /                         
      data fdmcs( 19, 2) /    .0000000017 8d0 /                         
      data fdmcs( 20, 2) /   -.0000000005 9d0 /                         
      data fdmcs( 21, 2) /    .0000000001 9d0 /                         
      data fdmcs( 22, 2) /   -.0000000000 66d0 /                        
c                                                                       
c series for f00b    on the interval  0.00000e-01 to  2.50000e-01       
c                                        with weighted error   2.19e-11 
c                                         log weighted error  10.66     
c                               significant figures required  10.66     
c                                    decimal places required  11.33     
c                                                                       
      data fdbcs(  1, 2) /   2.0021325464 9d0 /                         
      data fdbcs(  2, 2) /    .0018258034 26d0 /                        
      data fdbcs(  3, 2) /    .0011225105 54d0 /                        
      data fdbcs(  4, 2) /    .0004555439 28d0 /                        
      data fdbcs(  5, 2) /    .0000880185 91d0 /                        
      data fdbcs(  6, 2) /   -.0000137060 73d0 /                        
      data fdbcs(  7, 2) /   -.0000090015 63d0 /                        
      data fdbcs(  8, 2) /    .0000013512 84d0 /                        
      data fdbcs(  9, 2) /    .0000010449 37d0 /                        
      data fdbcs( 10, 2) /   -.0000003164 95d0 /                        
      data fdbcs( 11, 2) /   -.0000001071 28d0 /                        
      data fdbcs( 12, 2) /    .0000000783 47d0 /                        
      data fdbcs( 13, 2) /   -.0000000017 22d0 /                        
      data fdbcs( 14, 2) /   -.0000000148 37d0 /                        
      data fdbcs( 15, 2) /    .0000000055 82d0 /                        
      data fdbcs( 16, 2) /    .0000000010 85d0 /                        
      data fdbcs( 17, 2) /   -.0000000017 29d0 /                        
      data fdbcs( 18, 2) /    .0000000005 16d0 /                        
      data fdbcs( 19, 2) /    .0000000001 80d0 /                        
      data fdbcs( 20, 2) /   -.0000000002 22d0 /                        
      data fdbcs( 21, 2) /    .0000000000 691d0 /                       
      data fdbcs( 22, 2) /    .0000000000 219d0 /                       
c                                                                       
c series for f05s    on the interval  0.00000e-01 to  2.71828e+00       
c                                        with weighted error   2.87e-11 
c                                         log weighted error  10.54     
c                               significant figures required  10.38     
c                                    decimal places required  11.18     
c                                                                       
      data fdscs(  1, 3) /   1.3284029124d0 /                           
      data fdscs(  2, 3) /   -.1783849741 0d0 /                         
      data fdscs(  3, 3) /    .0338871501 3d0 /                         
      data fdscs(  4, 3) /   -.0074116985 5d0 /                         
      data fdscs(  5, 3) /    .0017528135 4d0 /                         
      data fdscs(  6, 3) /   -.0004358562 3d0 /                         
      data fdscs(  7, 3) /    .0001122556 1d0 /                         
      data fdscs(  8, 3) /   -.0000296748 6d0 /                         
      data fdscs(  9, 3) /    .0000080041 3d0 /                         
      data fdscs( 10, 3) /   -.0000021938 6d0 /                         
      data fdscs( 11, 3) /    .0000006092 5d0 /                         
      data fdscs( 12, 3) /   -.0000001710 4d0 /                         
      data fdscs( 13, 3) /    .0000000484 6d0 /                         
      data fdscs( 14, 3) /   -.0000000138 4d0 /                         
      data fdscs( 15, 3) /    .0000000039 8d0 /                         
      data fdscs( 16, 3) /   -.0000000011 5d0 /                         
      data fdscs( 17, 3) /    .0000000003 3d0 /                         
      data fdscs( 18, 3) /   -.0000000000 97d0 /                        
      data fdscs( 19, 3) /    .0000000000 28d0 /                        
c                                                                       
c series for f05m    on the interval  1.00000e+00 to  4.00000e+00       
c                                        with weighted error   2.90e-11 
c                                         log weighted error  10.54     
c                               significant figures required  10.52     
c                                    decimal places required  11.23     
c                                                                       
      data fdmcs(  1, 3) /   1.8324909204 1d0 /                         
      data fdmcs(  2, 3) /   -.2831869649 90d0 /                        
      data fdmcs(  3, 3) /    .1228778370 71d0 /                        
      data fdmcs(  4, 3) /   -.0473020885 11d0 /                        
      data fdmcs(  5, 3) /    .0172527816 16d0 /                        
      data fdmcs(  6, 3) /   -.0061561464 96d0 /                        
      data fdmcs(  7, 3) /    .0021769305 63d0 /                        
      data fdmcs(  8, 3) /   -.0007657618 44d0 /                        
      data fdmcs(  9, 3) /    .0002681487 29d0 /                        
      data fdmcs( 10, 3) /   -.0000935040 12d0 /                        
      data fdmcs( 11, 3) /    .0000324836 07d0 /                        
      data fdmcs( 12, 3) /   -.0000112489 79d0 /                        
      data fdmcs( 13, 3) /    .0000038849 42d0 /                        
      data fdmcs( 14, 3) /   -.0000013385 73d0 /                        
      data fdmcs( 15, 3) /    .0000004602 70d0 /                        
      data fdmcs( 16, 3) /   -.0000001579 79d0 /                        
      data fdmcs( 17, 3) /    .0000000541 36d0 /                        
      data fdmcs( 18, 3) /   -.0000000185 24d0 /                        
      data fdmcs( 19, 3) /    .0000000063 30d0 /                        
      data fdmcs( 20, 3) /   -.0000000021 60d0 /                        
      data fdmcs( 21, 3) /    .0000000007 36d0 /                        
      data fdmcs( 22, 3) /   -.0000000002 50d0 /                        
      data fdmcs( 23, 3) /    .0000000000 854d0 /                       
      data fdmcs( 24, 3) /   -.0000000000 290d0 /                       
c                                                                       
c series for f05b    on the interval  0.00000e-01 to  2.50000e-01       
c                                        with weighted error   2.38e-11 
c                                         log weighted error  10.62     
c                               significant figures required  10.46     
c                                    decimal places required  11.27     
c                                                                       
      data fdbcs(  1, 3) /   1.3737552319d0 /                           
      data fdbcs(  2, 3) /    .0271865732 7d0 /                         
      data fdbcs(  3, 3) /    .0071381409 2d0 /                         
      data fdbcs(  4, 3) /    .0001635361 7d0 /                         
      data fdbcs(  5, 3) /   -.0000117682 0d0 /                         
      data fdbcs(  6, 3) /   -.0000141643 9d0 /                         
      data fdbcs(  7, 3) /   -.0000002415 6d0 /                         
      data fdbcs(  8, 3) /    .0000012719 9d0 /                         
      data fdbcs(  9, 3) /   -.0000000281 9d0 /                         
      data fdbcs( 10, 3) /   -.0000001602 3d0 /                         
      data fdbcs( 11, 3) /    .0000000306 4d0 /                         
      data fdbcs( 12, 3) /    .0000000187 1d0 /                         
      data fdbcs( 13, 3) /   -.0000000101 5d0 /                         
      data fdbcs( 14, 3) /   -.0000000003 8d0 /                         
      data fdbcs( 15, 3) /    .0000000021 2d0 /                         
      data fdbcs( 16, 3) /   -.0000000007 1d0 /                         
      data fdbcs( 17, 3) /   -.0000000001 7d0 /                         
      data fdbcs( 18, 3) /    .0000000002 3d0 /                         
      data fdbcs( 19, 3) /   -.0000000000 70d0 /                        
      data fdbcs( 20, 3) /   -.0000000000 23d0 /                        
c                                                                       
c series for f10s    on the interval  0.00000e-01 to  2.71828e+00       
c                                        with weighted error   3.46e-11 
c                                         log weighted error  10.46     
c                               significant figures required  10.38     
c                                    decimal places required  11.09     
c                                                                       
      data fdscs(  1, 4) /   1.6098847156d0 /                           
      data fdscs(  2, 4) /   -.1624060718 4d0 /                         
      data fdscs(  3, 4) /    .0261468226 3d0 /                         
      data fdscs(  4, 4) /   -.0050782609 1d0 /                         
      data fdscs(  5, 4) /    .0010937471 2d0 /                         
      data fdscs(  6, 4) /   -.0002516893 5d0 /                         
      data fdscs(  7, 4) /    .0000606609 8d0 /                         
      data fdscs(  8, 4) /   -.0000151303 0d0 /                         
      data fdscs(  9, 4) /    .0000038751 7d0 /                         
      data fdscs( 10, 4) /   -.0000010136 9d0 /                         
      data fdscs( 11, 4) /    .0000002697 7d0 /                         
      data fdscs( 12, 4) /   -.0000000728 3d0 /                         
      data fdscs( 13, 4) /    .0000000199 0d0 /                         
      data fdscs( 14, 4) /   -.0000000054 9d0 /                         
      data fdscs( 15, 4) /    .0000000015 3d0 /                         
      data fdscs( 16, 4) /   -.0000000004 3d0 /                         
      data fdscs( 17, 4) /    .0000000001 2d0 /                         
      data fdscs( 18, 4) /   -.0000000000 34d0 /                        
c                                                                       
c series for f10m    on the interval  1.00000e+00 to  4.00000e+00       
c                                        with weighted error   3.92e-11 
c                                         log weighted error  10.41     
c                               significant figures required  10.44     
c                                    decimal places required  11.11     
c                                                                       
      data fdmcs(  1, 4) /   1.8823916245d0 /                           
      data fdmcs(  2, 4) /   -.4963882506 6d0 /                         
      data fdmcs(  3, 4) /    .2216981727 0d0 /                         
      data fdmcs(  4, 4) /   -.0903287142 0d0 /                         
      data fdmcs(  5, 4) /    .0352537623 9d0 /                         
      data fdmcs(  6, 4) /   -.0134370871 3d0 /                         
      data fdmcs(  7, 4) /    .0050410084 6d0 /                         
      data fdmcs(  8, 4) /   -.0018681458 3d0 /                         
      data fdmcs(  9, 4) /    .0006853986 5d0 /                         
      data fdmcs( 10, 4) /   -.0002493647 5d0 /                         
      data fdmcs( 11, 4) /    .0000900867 6d0 /                         
      data fdmcs( 12, 4) /   -.0000323503 7d0 /                         
      data fdmcs( 13, 4) /    .0000115572 5d0 /                         
      data fdmcs( 14, 4) /   -.0000041103 4d0 /                         
      data fdmcs( 15, 4) /    .0000014560 9d0 /                         
      data fdmcs( 16, 4) /   -.0000005140 2d0 /                         
      data fdmcs( 17, 4) /    .0000001808 9d0 /                         
      data fdmcs( 18, 4) /   -.0000000634 8d0 /                         
      data fdmcs( 19, 4) /    .0000000222 2d0 /                         
      data fdmcs( 20, 4) /   -.0000000077 6d0 /                         
      data fdmcs( 21, 4) /    .0000000027 0d0 /                         
      data fdmcs( 22, 4) /   -.0000000009 4d0 /                         
      data fdmcs( 23, 4) /    .0000000003 2d0 /                         
      data fdmcs( 24, 4) /   -.0000000001 1d0 /                         
      data fdmcs( 25, 4) /    .0000000000 39d0 /                        
c                                                                       
c series for f10b    on the interval  0.00000e-01 to  2.50000e-01       
c                                        with weighted error   3.24e-11 
c                                         log weighted error  10.49     
c                               significant figures required  10.22     
c                                    decimal places required  11.12     
c                                                                       
      data fdbcs(  1, 4) /   1.0766097168d0 /                           
      data fdbcs(  2, 4) /    .0509708968 7d0 /                         
      data fdbcs(  3, 4) /    .0125669010 0d0 /                         
      data fdbcs(  4, 4) /   -.0001333933 2d0 /                         
      data fdbcs(  5, 4) /   -.0000390220 0d0 /                         
      data fdbcs(  6, 4) /   -.0000033829 5d0 /                         
      data fdbcs(  7, 4) /    .0000018566 7d0 /                         
      data fdbcs(  8, 4) /    .0000003251 2d0 /                         
      data fdbcs(  9, 4) /   -.0000001931 9d0 /                         
      data fdbcs( 10, 4) /   -.0000000183 8d0 /                         
      data fdbcs( 11, 4) /    .0000000281 7d0 /                         
      data fdbcs( 12, 4) /   -.0000000030 6d0 /                         
      data fdbcs( 13, 4) /   -.0000000037 4d0 /                         
      data fdbcs( 14, 4) /    .0000000016 2d0 /                         
      data fdbcs( 15, 4) /    .0000000001 6d0 /                         
      data fdbcs( 16, 4) /   -.0000000003 7d0 /                         
      data fdbcs( 17, 4) /    .0000000001 1d0 /                         
      data fdbcs( 18, 4) /    .0000000000 32d0 /                        
c                                                                       
c series for f15s    on the interval  0.00000e-01 to  2.71828e+00       
c                                        with weighted error   5.17e-11 
c                                         log weighted error  10.29     
c                               significant figures required  10.34     
c                                    decimal places required  10.90     
c                                                                       
      data fdscs(  1, 5) /   2.2601818297d0 /                           
      data fdscs(  2, 5) /   -.1708722765 0d0 /                         
      data fdscs(  3, 5) /    .0233363666 4d0 /                         
      data fdscs(  4, 5) /   -.0040304134 4d0 /                         
      data fdscs(  5, 5) /    .0007916285 5d0 /                         
      data fdscs(  6, 5) /   -.0001687845 2d0 /                         
      data fdscs(  7, 5) /    .0000381078 6d0 /                         
      data fdscs(  8, 5) /   -.0000089765 6d0 /                         
      data fdscs(  9, 5) /    .0000021848 5d0 /                         
      data fdscs( 10, 5) /   -.0000005458 3d0 /                         
      data fdscs( 11, 5) /    .0000001393 0d0 /                         
      data fdscs( 12, 5) /   -.0000000361 8d0 /                         
      data fdscs( 13, 5) /    .0000000095 4d0 /                         
      data fdscs( 14, 5) /   -.0000000025 4d0 /                         
      data fdscs( 15, 5) /    .0000000006 8d0 /                         
      data fdscs( 16, 5) /   -.0000000001 8d0 /                         
      data fdscs( 17, 5) /    .0000000000 51d0 /                        
c                                                                       
c series for f15m    on the interval  1.00000e+00 to  4.00000e+00       
c                                        with weighted error   5.45e-11 
c                                         log weighted error  10.26     
c                               significant figures required  10.43     
c                                    decimal places required  10.97     
c                                                                       
      data fdmcs(  1, 5) /   2.2310484506d0 /                           
      data fdmcs(  2, 5) /   -.8435086752 5d0 /                         
      data fdmcs(  3, 5) /    .4030118348 2d0 /                         
      data fdmcs(  4, 5) /   -.1765850337 0d0 /                         
      data fdmcs(  5, 5) /    .0738231089 9d0 /                         
      data fdmcs(  6, 5) /   -.0299237390 3d0 /                         
      data fdmcs(  7, 5) /    .0118550148 7d0 /                         
      data fdmcs(  8, 5) /   -.0046128514 7d0 /                         
      data fdmcs(  9, 5) /    .0017688631 8d0 /                         
      data fdmcs( 10, 5) /   -.0006701511 6d0 /                         
      data fdmcs( 11, 5) /    .0002513292 1d0 /                         
      data fdmcs( 12, 5) /   -.0000934452 7d0 /                         
      data fdmcs( 13, 5) /    .0000344851 9d0 /                         
      data fdmcs( 14, 5) /   -.0000126440 2d0 /                         
      data fdmcs( 15, 5) /    .0000046095 2d0 /                         
      data fdmcs( 16, 5) /   -.0000016719 6d0 /                         
      data fdmcs( 17, 5) /    .0000006037 1d0 /                         
      data fdmcs( 18, 5) /   -.0000002171 0d0 /                         
      data fdmcs( 19, 5) /    .0000000777 9d0 /                         
      data fdmcs( 20, 5) /   -.0000000277 8d0 /                         
      data fdmcs( 21, 5) /    .0000000098 9d0 /                         
      data fdmcs( 22, 5) /   -.0000000035 1d0 /                         
      data fdmcs( 23, 5) /    .0000000012 4d0 /                         
      data fdmcs( 24, 5) /   -.0000000004 3d0 /                         
      data fdmcs( 25, 5) /    .0000000001 5d0 /                         
      data fdmcs( 26, 5) /   -.0000000000 54d0 /                        
c                                                                       
c series for f15b    on the interval  0.00000e-01 to  2.50000e-01       
c                                        with weighted error   2.21e-11 
c                                         log weighted error  10.66     
c                               significant figures required  10.32     
c                                    decimal places required  11.28     
c                                                                       
      data fdbcs(  1, 5) /    .9138458031 3d0 /                         
      data fdbcs(  2, 5) /    .0756461485 3d0 /                         
      data fdbcs(  3, 5) /    .0185325720 6d0 /                         
      data fdbcs(  4, 5) /   -.0002173856 5d0 /                         
      data fdbcs(  5, 5) /   -.0000237328 8d0 /                         
      data fdbcs(  6, 5) /    .0000042673 3d0 /                         
      data fdbcs(  7, 5) /    .0000012018 7d0 /                         
      data fdbcs(  8, 5) /   -.0000002038 1d0 /                         
      data fdbcs(  9, 5) /   -.0000000983 9d0 /                         
      data fdbcs( 10, 5) /    .0000000298 3d0 /                         
      data fdbcs( 11, 5) /    .0000000073 9d0 /                         
      data fdbcs( 12, 5) /   -.0000000054 3d0 /                         
      data fdbcs( 13, 5) /    .0000000001 9d0 /                         
      data fdbcs( 14, 5) /    .0000000008 2d0 /                         
      data fdbcs( 15, 5) /   -.0000000002 9d0 /                         
      data fdbcs( 16, 5) /   -.0000000000 48d0 /                        
      data fdbcs( 17, 5) /    .0000000000 77d0 /                        
      data fdbcs( 18, 5) /   -.0000000000 22d0 /                        
c                                                                       
c series for f20s    on the interval  0.00000e-01 to  2.71828e+00       
c                                        with weighted error   2.47e-11 
c                                         log weighted error  10.61     
c                               significant figures required  10.86     
c                                    decimal places required  11.22     
c                                                                       
      data fdscs(  1, 6) /   3.5445815749 6d0 /                         
      data fdscs(  2, 6) /   -.2001497509 36d0 /                        
      data fdscs(  3, 6) /    .0231937129 11d0 /                        
      data fdscs(  4, 6) /   -.0035654858 18d0 /                        
      data fdscs(  5, 6) /    .0006393090 63d0 /                        
      data fdscs(  6, 6) /   -.0001264180 87d0 /                        
      data fdscs(  7, 6) /    .0000267615 70d0 /                        
      data fdscs(  8, 6) /   -.0000059580 71d0 /                        
      data fdscs(  9, 6) /    .0000013790 87d0 /                        
      data fdscs( 10, 6) /   -.0000003292 55d0 /                        
      data fdscs( 11, 6) /    .0000000806 22d0 /                        
      data fdscs( 12, 6) /   -.0000000201 61d0 /                        
      data fdscs( 13, 6) /    .0000000051 32d0 /                        
      data fdscs( 14, 6) /   -.0000000013 26d0 /                        
      data fdscs( 15, 6) /    .0000000003 47d0 /                        
      data fdscs( 16, 6) /   -.0000000000 921d0 /                       
      data fdscs( 17, 6) /    .0000000000 246d0 /                       
c                                                                       
c series for f20m    on the interval  1.00000e+00 to  4.00000e+00       
c                                        with weighted error   2.78e-11 
c                                         log weighted error  10.56     
c                               significant figures required  10.91     
c                                    decimal places required  11.28     
c                                                                       
      data fdmcs(  1, 6) /   2.9777839001 0d0 /                         
      data fdmcs(  2, 6) /  -1.4577413536 2d0 /                         
      data fdmcs(  3, 6) /    .7528269512 35d0 /                        
      data fdmcs(  4, 6) /   -.3549647428 05d0 /                        
      data fdmcs(  5, 6) /    .1584014924 88d0 /                        
      data fdmcs(  6, 6) /   -.0680073485 74d0 /                        
      data fdmcs(  7, 6) /    .0283569566 67d0 /                        
      data fdmcs(  8, 6) /   -.0115545568 43d0 /                        
      data fdmcs(  9, 6) /    .0046209871 95d0 /                        
      data fdmcs( 10, 6) /   -.0018197198 20d0 /                        
      data fdmcs( 11, 6) /    .0007073370 31d0 /                        
      data fdmcs( 12, 6) /   -.0002719114 95d0 /                        
      data fdmcs( 13, 6) /    .0001035295 30d0 /                        
      data fdmcs( 14, 6) /   -.0000390900 34d0 /                        
      data fdmcs( 15, 6) /    .0000146509 87d0 /                        
      data fdmcs( 16, 6) /   -.0000054554 02d0 /                        
      data fdmcs( 17, 6) /    .0000020195 19d0 /                        
      data fdmcs( 18, 6) /   -.0000007436 80d0 /                        
      data fdmcs( 19, 6) /    .0000002725 59d0 /                        
      data fdmcs( 20, 6) /   -.0000000994 63d0 /                        
      data fdmcs( 21, 6) /    .0000000361 53d0 /                        
      data fdmcs( 22, 6) /   -.0000000130 94d0 /                        
      data fdmcs( 23, 6) /    .0000000047 26d0 /                        
      data fdmcs( 24, 6) /   -.0000000017 01d0 /                        
      data fdmcs( 25, 6) /    .0000000006 10d0 /                        
      data fdmcs( 26, 6) /   -.0000000002 18d0 /                        
      data fdmcs( 27, 6) /    .0000000000 780d0 /                       
      data fdmcs( 28, 6) /   -.0000000000 277d0 /                       
c                                                                       
c series for f20b    on the interval  0.00000e-01 to  2.50000e-01       
c                                        with weighted error   1.41e-11 
c                                         log weighted error  10.85     
c                               significant figures required  10.48     
c                                    decimal places required  11.47     
c                                                                       
      data fdbcs(  1, 6) /    .8211121274 8d0 /                         
      data fdbcs(  2, 6) /    .1030146856 2d0 /                         
      data fdbcs(  3, 6) /    .0258442758 9d0 /                         
      data fdbcs(  4, 6) /    .0000739478 7d0 /                         
      data fdbcs(  5, 6) /    .0000269632 3d0 /                         
      data fdbcs(  6, 6) /    .0000055393 5d0 /                         
      data fdbcs(  7, 6) /   -.0000000666 0d0 /                         
      data fdbcs(  8, 6) /   -.0000002863 2d0 /                         
      data fdbcs(  9, 6) /    .0000000098 7d0 /                         
      data fdbcs( 10, 6) /    .0000000250 2d0 /                         
      data fdbcs( 11, 6) /   -.0000000043 8d0 /                         
      data fdbcs( 12, 6) /   -.0000000022 7d0 /                         
      data fdbcs( 13, 6) /    .0000000011 2d0 /                         
      data fdbcs( 14, 6) /    .0000000000 40d0 /                        
      data fdbcs( 15, 6) /   -.0000000001 9d0 /                         
      data fdbcs( 16, 6) /    .0000000000 60d0 /                        
      data fdbcs( 17, 6) /    .0000000000 14d0 /                        
c                                                                       
c series for f25s    on the interval  0.00000e-01 to  2.71828e+00       
c                                        with weighted error   4.95e-11 
c                                         log weighted error  10.31     
c                               significant figures required  10.79     
c                                    decimal places required  10.91     
c                                                                       
      data fdscs(  1, 7) /   6.0776352655 5d0 /                         
      data fdscs(  2, 7) /   -.2553089335 90d0 /                        
      data fdscs(  3, 7) /    .0250962593 28d0 /                        
      data fdscs(  4, 7) /   -.0034359138 78d0 /                        
      data fdscs(  5, 7) /    .0005628501 70d0 /                        
      data fdscs(  6, 7) /   -.0001033045 44d0 /                        
      data fdscs(  7, 7) /    .0000205192 58d0 /                        
      data fdscs(  8, 7) /   -.0000043206 22d0 /                        
      data fdscs(  9, 7) /    .0000009516 32d0 /                        
      data fdscs( 10, 7) /   -.0000002172 43d0 /                        
      data fdscs( 11, 7) /    .0000000510 64d0 /                        
      data fdscs( 12, 7) /   -.0000000122 98d0 /                        
      data fdscs( 13, 7) /    .0000000030 23d0 /                        
      data fdscs( 14, 7) /   -.0000000007 56d0 /                        
      data fdscs( 15, 7) /    .0000000001 92d0 /                        
      data fdscs( 16, 7) /   -.0000000000 495d0 /                       
c                                                                       
c series for f25m    on the interval  1.00000e+00 to  4.00000e+00       
c                                        with weighted error   4.15e-11 
c                                         log weighted error  10.38     
c                               significant figures required  10.96     
c                                    decimal places required  11.11     
c                                                                       
      data fdmcs(  1, 7) /   4.4112295532 9d0 /                         
      data fdmcs(  2, 7) /  -2.6064119236 9d0 /                         
      data fdmcs(  3, 7) /   1.4541147675 7d0 /                         
      data fdmcs(  4, 7) /   -.7348922045 75d0 /                        
      data fdmcs(  5, 7) /    .3485252773 58d0 /                        
      data fdmcs(  6, 7) /   -.1579050700 24d0 /                        
      data fdmcs(  7, 7) /    .0690927288 72d0 /                        
      data fdmcs(  8, 7) /   -.0294110574 69d0 /                        
      data fdmcs(  9, 7) /    .0122427694 58d0 /                        
      data fdmcs( 10, 7) /   -.0050026362 88d0 /                        
      data fdmcs( 11, 7) /    .0020124730 44d0 /                        
      data fdmcs( 12, 7) /   -.0007988327 90d0 /                        
      data fdmcs( 13, 7) /    .0003134423 09d0 /                        
      data fdmcs( 14, 7) /   -.0001217494 74d0 /                        
      data fdmcs( 15, 7) /    .0000468708 54d0 /                        
      data fdmcs( 16, 7) /   -.0000179017 70d0 /                        
      data fdmcs( 17, 7) /    .0000067890 45d0 /                        
      data fdmcs( 18, 7) /   -.0000025582 83d0 /                        
      data fdmcs( 19, 7) /    .0000009584 71d0 /                        
      data fdmcs( 20, 7) /   -.0000003572 13d0 /                        
      data fdmcs( 21, 7) /    .0000001324 92d0 /                        
      data fdmcs( 22, 7) /   -.0000000489 26d0 /                        
      data fdmcs( 23, 7) /    .0000000179 94d0 /                        
      data fdmcs( 24, 7) /   -.0000000065 93d0 /                        
      data fdmcs( 25, 7) /    .0000000024 07d0 /                        
      data fdmcs( 26, 7) /   -.0000000008 76d0 /                        
      data fdmcs( 27, 7) /    .0000000003 17d0 /                        
      data fdmcs( 28, 7) /   -.0000000001 15d0 /                        
      data fdmcs( 29, 7) /    .0000000000 415d0 /                       
c                                                                       
c series for f25b    on the interval  0.00000e-01 to  2.50000e-01       
c                                        with weighted error   4.96e-11 
c                                         log weighted error  10.30     
c                               significant figures required   9.92     
c                                    decimal places required  10.91     
c                                                                       
      data fdbcs(  1, 7) /    .7721541990 3d0 /                         
      data fdbcs(  2, 7) /    .1348979022 5d0 /                         
      data fdbcs(  3, 7) /    .0353565117 1d0 /                         
      data fdbcs(  4, 7) /    .0009476728 1d0 /                         
      data fdbcs(  5, 7) /    .0001277291 1d0 /                         
      data fdbcs(  6, 7) /    .0000007218 8d0 /                         
      data fdbcs(  7, 7) /   -.0000009206 8d0 /                         
      data fdbcs(  8, 7) /   -.0000001154 7d0 /                         
      data fdbcs(  9, 7) /    .0000000580 6d0 /                         
      data fdbcs( 10, 7) /    .0000000047 4d0 /                         
      data fdbcs( 11, 7) /   -.0000000060 8d0 /                         
      data fdbcs( 12, 7) /    .0000000005 1d0 /                         
      data fdbcs( 13, 7) /    .0000000006 5d0 /                         
      data fdbcs( 14, 7) /   -.0000000002 4d0 /                         
      data fdbcs( 15, 7) /   -.0000000000 28d0 /                        
      data fdbcs( 16, 7) /    .0000000000 49d0 /                        
c                                                                       
c series for f30s    on the interval  0.00000e-01 to  2.71828e+00       
c                                        with weighted error   2.87e-11 
c                                         log weighted error  10.54     
c                               significant figures required  11.29     
c                                    decimal places required  11.14     
c                                                                       
      data fdscs(  1, 8) /  11.2341939777d0 /                           
      data fdscs(  2, 8) /   -.3495789894 02d0 /                        
      data fdscs(  3, 8) /    .0291275872 60d0 /                        
      data fdscs(  4, 8) /   -.0035525827 96d0 /                        
      data fdscs(  5, 8) /    .0005319821 79d0 /                        
      data fdscs(  6, 8) /   -.0000906823 61d0 /                        
      data fdscs(  7, 8) /    .0000169110 38d0 /                        
      data fdscs(  8, 8) /   -.0000033697 23d0 /                        
      data fdscs(  9, 8) /    .0000007066 16d0 /                        
      data fdscs( 10, 8) /   -.0000001543 15d0 /                        
      data fdscs( 11, 8) /    .0000000348 35d0 /                        
      data fdscs( 12, 8) /   -.0000000080 83d0 /                        
      data fdscs( 13, 8) /    .0000000019 20d0 /                        
      data fdscs( 14, 8) /   -.0000000004 65d0 /                        
      data fdscs( 15, 8) /    .0000000001 14d0 /                        
      data fdscs( 16, 8) /   -.0000000000 287d0 /                       
c                                                                       
c series for f30m    on the interval  1.00000e+00 to  4.00000e+00       
c                                        with weighted error   2.33e-11 
c                                         log weighted error  10.63     
c                               significant figures required  11.47     
c                                    decimal places required  11.38     
c                                                                       
      data fdmcs(  1, 8) /   7.1719763668 6d0 /                         
      data fdmcs(  2, 8) /  -4.8571185185 4d0 /                         
      data fdmcs(  3, 8) /   2.9113197795 3d0 /                         
      data fdmcs(  4, 8) /  -1.5684642300 9d0 /                         
      data fdmcs(  5, 8) /    .7869998941 23d0 /                        
      data fdmcs(  6, 8) /   -.3749544936 90d0 /                        
      data fdmcs(  7, 8) /    .1716880079 87d0 /                        
      data fdmcs(  8, 8) /   -.0761760305 76d0 /                        
      data fdmcs(  9, 8) /    .0329421355 00d0 /                        
      data fdmcs( 10, 8) /   -.0139450242 81d0 /                        
      data fdmcs( 11, 8) /    .0057976754 88d0 /                        
      data fdmcs( 12, 8) /   -.0023734227 03d0 /                        
      data fdmcs( 13, 8) /    .0009586830 99d0 /                        
      data fdmcs( 14, 8) /   -.0003827164 22d0 /                        
      data fdmcs( 15, 8) /    .0001512084 34d0 /                        
      data fdmcs( 16, 8) /   -.0000591925 75d0 /                        
      data fdmcs( 17, 8) /    .0000229809 46d0 /                        
      data fdmcs( 18, 8) /   -.0000088559 00d0 /                        
      data fdmcs( 19, 8) /    .0000033897 35d0 /                        
      data fdmcs( 20, 8) /   -.0000012895 26d0 /                        
      data fdmcs( 21, 8) /    .0000004878 14d0 /                        
      data fdmcs( 22, 8) /   -.0000001835 85d0 /                        
      data fdmcs( 23, 8) /    .0000000687 64d0 /                        
      data fdmcs( 24, 8) /   -.0000000256 43d0 /                        
      data fdmcs( 25, 8) /    .0000000095 24d0 /                        
      data fdmcs( 26, 8) /   -.0000000035 23d0 /                        
      data fdmcs( 27, 8) /    .0000000012 99d0 /                        
      data fdmcs( 28, 8) /   -.0000000004 77d0 /                        
      data fdmcs( 29, 8) /    .0000000001 74d0 /                        
      data fdmcs( 30, 8) /   -.0000000000 638d0 /                       
      data fdmcs( 31, 8) /    .0000000000 232d0 /                       
c                                                                       
c series for f30b    on the interval  0.00000e-01 to  2.50000e-01       
c                                        with weighted error   5.48e-11 
c                                         log weighted error  10.26     
c                               significant figures required   9.88     
c                                    decimal places required  10.85     
c                                                                       
      data fdbcs(  1, 8) /    .7554309639 2d0 /                         
      data fdbcs(  2, 8) /    .1734863060 3d0 /                         
      data fdbcs(  3, 8) /    .0481579481 5d0 /                         
      data fdbcs(  4, 8) /    .0027149874 6d0 /                         
      data fdbcs(  5, 8) /    .0003217541 9d0 /                         
      data fdbcs(  6, 8) /   -.0000071412 9d0 /                         
      data fdbcs(  7, 8) /   -.0000009676 6d0 /                         
      data fdbcs(  8, 8) /    .0000001160 1d0 /                         
      data fdbcs(  9, 8) /    .0000000450 4d0 /                         
      data fdbcs( 10, 8) /   -.0000000103 6d0 /                         
      data fdbcs( 11, 8) /   -.0000000025 9d0 /                         
      data fdbcs( 12, 8) /    .0000000014 6d0 /                         
      data fdbcs( 13, 8) /   -.0000000000 04d0 /                        
      data fdbcs( 14, 8) /   -.0000000001 8d0 /                         
      data fdbcs( 15, 8) /    .0000000000 54d0 /                        
c                                                                       
c series for f35s    on the interval  0.00000e-01 to  2.71828e+00       
c                                        with weighted error   7.30e-11 
c                                         log weighted error  10.14     
c                               significant figures required  11.18     
c                                    decimal places required  10.72     
c                                                                       
      data fdscs(  1, 9) /  22.1653046970d0 /                           
      data fdscs(  2, 9) /   -.5086526539 69d0 /                        
      data fdscs(  3, 9) /    .0358871327 23d0 /                        
      data fdscs(  4, 9) /   -.0038993959 73d0 /                        
      data fdscs(  5, 9) /    .0005339699 07d0 /                        
      data fdscs(  6, 9) /   -.0000845770 08d0 /                        
      data fdscs(  7, 9) /    .0000148157 47d0 /                        
      data fdscs(  8, 9) /   -.0000027951 08d0 /                        
      data fdscs(  9, 9) /    .0000005582 82d0 /                        
      data fdscs( 10, 9) /   -.0000001166 84d0 /                        
      data fdscs( 11, 9) /    .0000000253 06d0 /                        
      data fdscs( 12, 9) /   -.0000000056 60d0 /                        
      data fdscs( 13, 9) /    .0000000012 99d0 /                        
      data fdscs( 14, 9) /   -.0000000003 05d0 /                        
      data fdscs( 15, 9) /    .0000000000 730d0 /                       
c                                                                       
c series for f35m    on the interval  1.00000e+00 to  4.00000e+00       
c                                        with weighted error   3.73e-11 
c                                         log weighted error  10.43     
c                               significant figures required  11.56     
c                                    decimal places required  11.18     
c                                                                       
      data fdmcs(  1, 9) /  12.6701567503 6d0 /                         
      data fdmcs(  2, 9) /  -9.4636920164 00d0 /                        
      data fdmcs(  3, 9) /   6.0480654287 69d0 /                        
      data fdmcs(  4, 9) /  -3.4530339209 92d0 /                        
      data fdmcs(  5, 9) /   1.8250457226 69d0 /                        
      data fdmcs(  6, 9) /   -.9112870648 186d0 /                       
      data fdmcs(  7, 9) /    .4354953493 280d0 /                       
      data fdmcs(  8, 9) /   -.2009635658 884d0 /                       
      data fdmcs(  9, 9) /    .0901210173 526d0 /                       
      data fdmcs( 10, 9) /   -.0394612435 160d0 /                       
      data fdmcs( 11, 9) /    .0169328410 948d0 /                       
      data fdmcs( 12, 9) /   -.0071407017 340d0 /                       
      data fdmcs( 13, 9) /    .0029661522 591d0 /                       
      data fdmcs( 14, 9) /   -.0012158829 523d0 /                       
      data fdmcs( 15, 9) /    .0004926051 670d0 /                       
      data fdmcs( 16, 9) /   -.0001975006 123d0 /                       
      data fdmcs( 17, 9) /    .0000784453 353d0 /                       
      data fdmcs( 18, 9) /   -.0000308953 181d0 /                       
      data fdmcs( 19, 9) /    .0000120749 876d0 /                       
      data fdmcs( 20, 9) /   -.0000046864 594d0 /                       
      data fdmcs( 21, 9) /    .0000018072 775d0 /                       
      data fdmcs( 22, 9) /   -.0000006928 714d0 /                       
      data fdmcs( 23, 9) /    .0000002641 967d0 /                       
      data fdmcs( 24, 9) /   -.0000001002 365d0 /                       
      data fdmcs( 25, 9) /    .0000000378 535d0 /                       
      data fdmcs( 26, 9) /   -.0000000142 333d0 /                       
      data fdmcs( 27, 9) /    .0000000053 303d0 /                       
      data fdmcs( 28, 9) /   -.0000000019 887d0 /                       
      data fdmcs( 29, 9) /    .0000000007 393d0 /                       
      data fdmcs( 30, 9) /   -.0000000002 739d0 /                       
      data fdmcs( 31, 9) /    .0000000001 011d0 /                       
      data fdmcs( 32, 9) /   -.0000000000 372d0 /                       
c                                                                       
c series for f35b    on the interval  0.00000e-01 to  2.50000e-01       
c                                        with weighted error   5.48e-11 
c                                         log weighted error  10.26     
c                               significant figures required   9.91     
c                                    decimal places required  10.85     
c                                                                       
      data fdbcs(  1, 9) /    .7665702804 4d0 /                         
      data fdbcs(  2, 9) /    .2216671315 0d0 /                         
      data fdbcs(  3, 9) /    .0657600143 6d0 /                         
      data fdbcs(  4, 9) /    .0058625466 3d0 /                         
      data fdbcs(  5, 9) /    .0006969431 7d0 /                         
      data fdbcs(  6, 9) /   -.0000101109 1d0 /                         
      data fdbcs(  7, 9) /   -.0000000660 4d0 /                         
      data fdbcs(  8, 9) /    .0000002536 7d0 /                         
      data fdbcs(  9, 9) /   -.0000000021 4d0 /                         
      data fdbcs( 10, 9) /   -.0000000132 6d0 /                         
      data fdbcs( 11, 9) /    .0000000015 0d0 /                         
      data fdbcs( 12, 9) /    .0000000009 5d0 /                         
      data fdbcs( 13, 9) /   -.0000000003 4d0 /                         
      data fdbcs( 14, 9) /   -.0000000000 30d0 /                        
      data fdbcs( 15, 9) /    .0000000000 54d0 /                        
c                                                                       
c series for f40s    on the interval  0.00000e-01 to  2.71828e+00       
c                                        with weighted error   4.91e-11 
c                                         log weighted error  10.31     
c                               significant figures required  11.67     
c                                    decimal places required  10.90     
c                                                                       
      data fdscs(  1,10) /  46.3350918683 9d0 /                         
      data fdscs(  2,10) /   -.7807026145 628d0 /                       
      data fdscs(  3,10) /    .0465789224 743d0 /                       
      data fdscs(  4,10) /   -.0045080435 976d0 /                       
      data fdscs(  5,10) /    .0005646381 627d0 /                       
      data fdscs(  6,10) /   -.0000831331 627d0 /                       
      data fdscs(  7,10) /    .0000136850 757d0 /                       
      data fdscs(  8,10) /   -.0000024454 133d0 /                       
      data fdscs(  9,10) /    .0000004654 205d0 /                       
      data fdscs( 10,10) /   -.0000000931 320d0 /                       
      data fdscs( 11,10) /    .0000000194 128d0 /                       
      data fdscs( 12,10) /   -.0000000041 863d0 /                       
      data fdscs( 13,10) /    .0000000009 290d0 /                       
      data fdscs( 14,10) /   -.0000000002 113d0 /                       
      data fdscs( 15,10) /    .0000000000 491d0 /                       
c                                                                       
c series for f40m    on the interval  1.00000e+00 to  4.00000e+00       
c                                        with weighted error   6.13e-11 
c                                         log weighted error  10.21     
c                               significant figures required  11.66     
c                                    decimal places required  10.97     
c                                                                       
      data fdmcs(  1,10) /  24.0980879457 2d0 /                         
      data fdmcs(  2,10) / -19.2973238247 9d0 /                         
      data fdmcs(  3,10) /  13.0411335433 2d0 /                         
      data fdmcs(  4,10) /  -7.8442177530 69d0 /                        
      data fdmcs(  5,10) /   4.3484777309 21d0 /                        
      data fdmcs(  6,10) /  -2.2682065310 65d0 /                        
      data fdmcs(  7,10) /   1.1283901506 72d0 /                        
      data fdmcs(  8,10) /   -.5404257776 187d0 /                       
      data fdmcs(  9,10) /    .2508755478 873d0 /                       
      data fdmcs( 10,10) /   -.1134573260 212d0 /                       
      data fdmcs( 11,10) /    .0501830996 299d0 /                       
      data fdmcs( 12,10) /   -.0217756382 802d0 /                       
      data fdmcs( 13,10) /    .0092927921 972d0 /                       
      data fdmcs( 14,10) /   -.0039080375 664d0 /                       
      data fdmcs( 15,10) /    .0016223028 722d0 /                       
      data fdmcs( 16,10) /   -.0006656886 564d0 /                       
      data fdmcs( 17,10) /    .0002703262 001d0 /                       
      data fdmcs( 18,10) /   -.0001087475 762d0 /                       
      data fdmcs( 19,10) /    .0000433751 765d0 /                       
      data fdmcs( 20,10) /   -.0000171663 866d0 /                       
      data fdmcs( 21,10) /    .0000067455 409d0 /                       
      data fdmcs( 22,10) /   -.0000026333 256d0 /                       
      data fdmcs( 23,10) /    .0000010217 923d0 /                       
      data fdmcs( 24,10) /   -.0000003942 636d0 /                       
      data fdmcs( 25,10) /    .0000001513 391d0 /                       
      data fdmcs( 26,10) /   -.0000000578 112d0 /                       
      data fdmcs( 27,10) /    .0000000219 841d0 /                       
      data fdmcs( 28,10) /   -.0000000083 247d0 /                       
      data fdmcs( 29,10) /    .0000000031 398d0 /                       
      data fdmcs( 30,10) /   -.0000000011 798d0 /                       
      data fdmcs( 31,10) /    .0000000004 418d0 /                       
      data fdmcs( 32,10) /   -.0000000001 648d0 /                       
      data fdmcs( 33,10) /    .0000000000 613d0 /                       
c                                                                       
c series for f40b    on the interval  0.00000e-01 to  2.50000e-01       
c                                        with weighted error   1.68e-11 
c                                         log weighted error  10.77     
c                               significant figures required  10.47     
c                                    decimal places required  11.36     
c                                                                       
      data fdbcs(  1,10) /    .8056894147 6d0 /                         
      data fdbcs(  2,10) /    .2834447403 7d0 /                         
      data fdbcs(  3,10) /    .0903522185 7d0 /                         
      data fdbcs(  4,10) /    .0111606016 1d0 /                         
      data fdbcs(  5,10) /    .0014164744 6d0 /                         
      data fdbcs(  6,10) /    .0000100892 5d0 /                         
      data fdbcs(  7,10) /    .0000022449 5d0 /                         
      data fdbcs(  8,10) /    .0000001741 4d0 /                         
      data fdbcs(  9,10) /   -.0000000486 2d0 /                         
      data fdbcs( 10,10) /   -.0000000054 1d0 /                         
      data fdbcs( 11,10) /    .0000000035 1d0 /                         
      data fdbcs( 12,10) /   -.0000000000 82d0 /                        
      data fdbcs( 13,10) /   -.0000000003 1d0 /                         
      data fdbcs( 14,10) /    .0000000000 81d0 /                        
      data fdbcs( 15,10) /    .0000000000 16d0 /                        
c                                                                       
      data nscs / 21, 20, 19, 18, 17, 17, 16, 16, 15, 15 /              
      data nmcs / 21, 22, 24, 25, 26, 28, 29, 31, 32, 33 /              
      data nbcs / 26, 22, 20, 18, 18, 17, 16, 15, 15, 15 /              
c                                                                       
      data exp1, alfsml, alfbig, alfmax / 13*0.0 /                      
c                                                                       
      if (exp1.ne.0.0) go to 20                                         
      exp1 = 2.0/exp(1d0)                                               
c                                                                       
      alfsml = dlog (d1mach(1)/0.8863)                                  
      alfbig = dexp (dmin1 (-dlog(d1mach(1)),dlog(d1mach(2)))-dlog(2d0)) 
c                                                                       
      alfmax(1) = d1mach(2)                                             
      alfmax(2) = d1mach(2)                                             
      do 10 ndx=3,10                                                    
        xk = (ndx-2)*0.5                                                
        alfmax(ndx) = d1mach(2)**(1.0/(xk+1.0)) * 0.99                  
 10   continue                                                          
c                                                                       
 20   ndx = (xnu+1.0)*2.0 + 0.01                                        
      if (dabs(xnu-dfloat(ndx-2)*0.5).gt.1.0e-5) then
        write(6,*) ' fd: xnu must be integer multiple of 0.5' 
        stop '  fd:1'
      end if   
      if (ndx.lt.1 .or. ndx.gt.10) then
        write(6,*) ' fd: xnu is lt -0.5 or gt 4.0'               
        stop '  fd:2'
      end if                                                                       
                                                               
      if (alpha.lt.alfsml) then
         fd = 0.0                                           
         return 
      end if                                      
c                                                                       
      if (alpha.gt.alfmax(ndx)) THEN
         write(6,*) '  fd: alpha so big fd overflows'
         stop ' fd:3'
      end if         
c                                                                       
      if (ndx.ne.2) go to 30                                            
c                                                                       
c calculate the fermi-dirac function for xnu = 0                        
c                                                                       
      if (alpha.lt.10.0) fd = alnrel (exp(alpha))                       
      if (alpha.ge.10.0) fd = alpha + alnrel (exp(-alpha))              
      return                                                            
c                                                                       
 30   if (alpha.ge.1.0) go to 40                                        
      expalf = dexp (alpha)                                              
      fd = expalf * csevl (expalf*exp1-1.0, fdscs(1,ndx), nscs(ndx))    
      return                                                            
c                                                                       
 40   if (alpha.ge.4.0) go to 50                                        
      fd = alpha**(xnu+1.0) * csevl ((alpha-2.5)/1.5, fdmcs(1,ndx),     
     1  nmcs(ndx))                                                      
      return                                                            
c                                                                       
 50   alfi = 0.0                                                        
      if (alpha.lt.alfbig) alfi = 1.0/alpha                             
      fd = alpha**(xnu+1.0) * csevl ((alfi-0.125)*8.0, fdbcs(1,ndx),    
     1  nbcs(ndx))                                                      
      return                                                            
c                                                                       
      end                                                               

      function alnrel(x)  
      implicit doubleprecision(a-h,o-z)                                              
c***begin prologue  alnrel                                              
c***date written   770401   (yymmdd)                                    
c***revision date  820801   (yymmdd)                                    
c***category no.  c4b                                                   
c***keywords  elementary function,logarithm,relative                    
c***author  fullerton, w., (lanl)                                       
c***purpose  evaluates ln(1+x) accurate in the sense of relative error. 
c***description                                                         
c                                                                       
c alnrel(x) evaluates ln(1+x) accurately in the sense of relative       
c error when x is very small.  this routine must be used to             
c maintain relative error accuracy whenever x is small and              
c accurately known.                                                     
c                                                                       
c series for alnr       on the interval -3.75000d-01 to  3.75000d-01    
c                                        with weighted error   1.93e-17 
c                                         log weighted error  16.72     
c                               significant figures required  16.44     
c                                    decimal places required  17.40     
c***references  (none)                                                  
c***routines called  csevl,inits,d1mach                            
c***end prologue  alnrel                                                
      dimension alnrcs(23)                                              
      data alnrcs( 1) /   1.0378693562 743770d0 /                       
      data alnrcs( 2) /   -.1336430150 4908918d0 /                      
      data alnrcs( 3) /    .0194082491 35520563d0 /                     
      data alnrcs( 4) /   -.0030107551 12753577d0 /                     
      data alnrcs( 5) /    .0004869461 47971548d0 /                     
      data alnrcs( 6) /   -.0000810548 81893175d0 /                     
      data alnrcs( 7) /    .0000137788 47799559d0 /                     
      data alnrcs( 8) /   -.0000023802 21089435d0 /                     
      data alnrcs( 9) /    .0000004164 04162138d0 /                     
      data alnrcs(10) /   -.0000000735 95828378d0 /                     
      data alnrcs(11) /    .0000000131 17611876d0 /                     
      data alnrcs(12) /   -.0000000023 54670931d0 /                     
      data alnrcs(13) /    .0000000004 25227732d0 /                     
      data alnrcs(14) /   -.0000000000 77190894d0 /                     
      data alnrcs(15) /    .0000000000 14075746d0 /                     
      data alnrcs(16) /   -.0000000000 02576907d0 /                     
      data alnrcs(17) /    .0000000000 00473424d0 /                     
      data alnrcs(18) /   -.0000000000 00087249d0 /                     
      data alnrcs(19) /    .0000000000 00016124d0 /                     
      data alnrcs(20) /   -.0000000000 00002987d0 /                     
      data alnrcs(21) /    .0000000000 00000554d0 /                     
      data alnrcs(22) /   -.0000000000 00000103d0 /                     
      data alnrcs(23) /    .0000000000 00000019d0 /                     
      data nlnrel, xmin /0, 0./                                         
c***first executable statement  alnrel                                  
      if (nlnrel.ne.0) go to 10                                         
      nlnrel = inits (alnrcs, 23, 0.1*d1mach(3))                        
      xmin = -1.0 + dsqrt(d1mach(4))                                     
c                                                                       
 10   if (x.le.(-1.0)) then
        write(6,*) '  alnrel:  x is le -1'
        stop ' alnrel:1'
      end if  
      if (x.lt.xmin) then
         write(6,*) ' alnrel: answer lt half precision'
         write(6,*) ' beause x too near -1'
         stop '  alnrel:2'
      end if                             
c                                                                       
      if (abs(x).le.0.375) alnrel = x*(1. -                             
     1  x*csevl (x/.375, alnrcs, nlnrel))                               
      if (abs(x).gt.0.375) alnrel = dlog (1d0+x)                        
c                                                                       
      return                                                            
      end                                                               

      function csevl(x,cs,n) 
      implicit doubleprecision(a-h,o-z)                                           
c***begin prologue  csevl                                               
c***date written   770401   (yymmdd)                                    
c***revision date  820801   (yymmdd)                                    
c***category no.  c3a2                                                  
c***keywords  chebyshev,fnlib,special function                          
c***author  fullerton, w., (lanl)                                       
c***purpose  evaluate the n-term chebyshev series cs at x.              
c***description                                                         
c                                                                       
c evaluate the n-term chebyshev series cs at x.  adapted from           
c r. broucke, algorithm 446, c.a.c.m., 16, 254 (1973). also see fox     
c and parker, chebyshev polynomials in numerical analysis, oxford press 
c page 56.                                                              
c                                                                       
c       input arguments --                                              
c x    value at which the series is to be evaluated.                    
c cs   array of n terms of a chebyshev series.  in eval-                
c      uating cs, only half the first coefficient is summed.            
c n    number of terms in array cs.                                     
c***references  (none)                                                                                             
c***end prologue  csevl                                                 
c                                                                       
       dimension cs(1)  
                                                
c***first executable statement  csevl                                   
       if(n.lt.1) then
           write(6,*) 'csevl: number of terms le 0'
           stop  ' csevl:1' 
       end if 
       if(n.gt.1000) then 
           write(6,*) 'csevl: number of terms gt 1000'     
           stop  ' csevl:2'
       end if                                                        
       if (x.lt. -1.0 .or. x.gt. 1.0) then
          write(6,*) 'csevl:   x outside ( 1,+1)'
          stop ' csevl:3'
       end if                                              
c                                                                       
       b1=0.                                                            
       b0=0.                                                            
       twox=2.*x                                                        
       do 10 i=1,n                                                      
       b2=b1                                                            
       b1=b0                                                            
       ni=n+1-i                                                         
       b0=twox*b1-b2+cs(ni)                                             
 10    continue                                                         
c                                                                       
       csevl = 0.5 * (b0-b2)                                            
c                                                                       
       return                                                           
      end                                                               

      function inits(os,nos,eta) 
      implicit doubleprecision(a-h,o-z)                                       
c***begin prologue  inits                                               
c***date written   770401   (yymmdd)                                    
c***revision date  820801   (yymmdd)                                    
c***category no.  c3a2                                                  
c***keywords  initialize,orthogonal series,special function             
c***author  fullerton, w., (lanl)                                       
c***purpose  initializes an orthogonal series so that it defines the    
c            number of terms to carry in the series to meet a specified 
c            error.                                                     
c***description                                                         
c                                                                       
c initialize the orthogonal series so that inits is the number of terms 
c needed to insure the error is no larger than eta.  ordinarily, eta    
c will be chosen to be one-tenth machine precision.                     
c                                                                       
c             input arguments --                                        
c os     array of nos coefficients in an orthogonal series.             
c nos    number of coefficients in os.                                  
c eta    requested accuracy of series.                                  
c***references  (none)                                                  
                                           
c***end prologue  inits                                                 
      dimension os(nos)                                                 
c***first executable statement  inits                                   
      if (nos.lt.1) then
       write(6,*) 'inits: number of coefficients lt 1'  
       stop ' inits:1'
      endif                                                        
c                                                                       
      err = 0.                                                          
      do 10 ii=1,nos                                                    
        i = nos + 1 - ii                                                
        err = err + abs(os(i))                                          
        if (err.gt.eta) go to 20                                        
 10   continue                                                          
c                                                                       
 20   if (i.eq.nos) write(6,*) ' inits: eta may be too small'    
                                                          
      inits = i                                                         
c                                                                       
      return                                                            
      end  
                                                             
      DOUBLE PRECISION FUNCTION D1MACH(I)
      INTEGER I
C
C  DOUBLE-PRECISION MACHINE CONSTANTS
C  D1MACH( 1) = B**(EMIN-1), THE SMALLEST POSITIVE MAGNITUDE.
C  D1MACH( 2) = B**EMAX*(1 - B**(-T)), THE LARGEST MAGNITUDE.
C  D1MACH( 3) = B**(-T), THE SMALLEST RELATIVE SPACING.
C  D1MACH( 4) = B**(1-T), THE LARGEST RELATIVE SPACING.
C  D1MACH( 5) = LOG10(B)
C
      INTEGER SMALL(2)
      INTEGER LARGE(2)
      INTEGER RIGHT(2)
      INTEGER DIVER(2)
      INTEGER LOG10(2)
      INTEGER SC, CRAY1(38), J
      COMMON /D9MACH/ CRAY1
      SAVE SMALL, LARGE, RIGHT, DIVER, LOG10, SC
      DOUBLE PRECISION DMACH(5)
      EQUIVALENCE (DMACH(1),SMALL(1))
      EQUIVALENCE (DMACH(2),LARGE(1))
      EQUIVALENCE (DMACH(3),RIGHT(1))
      EQUIVALENCE (DMACH(4),DIVER(1))
      EQUIVALENCE (DMACH(5),LOG10(1))
C  THIS VERSION ADAPTS AUTOMATICALLY TO MOST CURRENT MACHINES.
C  R1MACH CAN HANDLE AUTO-DOUBLE COMPILING, BUT THIS VERSION OF
C  D1MACH DOES NOT, BECAUSE WE DO NOT HAVE QUAD CONSTANTS FOR
C  MANY MACHINES YET.
C  TO COMPILE ON OLDER MACHINES, ADD A C IN COLUMN 1
C  ON THE NEXT LINE
      DATA SC/0/
C  AND REMOVE THE C FROM COLUMN 1 IN ONE OF THE SECTIONS BELOW.
C  CONSTANTS FOR EVEN OLDER MACHINES CAN BE OBTAINED BY
C          mail netlib@research.bell-labs.com
C          send old1mach from blas
C  PLEASE SEND CORRECTIONS TO dmg OR ehg@bell-labs.com.
C
C     MACHINE CONSTANTS FOR THE HONEYWELL DPS 8/70 SERIES.
C      DATA SMALL(1),SMALL(2) / O402400000000, O000000000000 /
C      DATA LARGE(1),LARGE(2) / O376777777777, O777777777777 /
C      DATA RIGHT(1),RIGHT(2) / O604400000000, O000000000000 /
C      DATA DIVER(1),DIVER(2) / O606400000000, O000000000000 /
C      DATA LOG10(1),LOG10(2) / O776464202324, O117571775714 /, SC/987/
C
C     MACHINE CONSTANTS FOR PDP-11 FORTRANS SUPPORTING
C     32-BIT INTEGERS.
C      DATA SMALL(1),SMALL(2) /    8388608,           0 /
C      DATA LARGE(1),LARGE(2) / 2147483647,          -1 /
C      DATA RIGHT(1),RIGHT(2) /  612368384,           0 /
C      DATA DIVER(1),DIVER(2) /  620756992,           0 /
C      DATA LOG10(1),LOG10(2) / 1067065498, -2063872008 /, SC/987/
C
C     MACHINE CONSTANTS FOR THE UNIVAC 1100 SERIES.
C      DATA SMALL(1),SMALL(2) / O000040000000, O000000000000 /
C      DATA LARGE(1),LARGE(2) / O377777777777, O777777777777 /
C      DATA RIGHT(1),RIGHT(2) / O170540000000, O000000000000 /
C      DATA DIVER(1),DIVER(2) / O170640000000, O000000000000 /
C      DATA LOG10(1),LOG10(2) / O177746420232, O411757177572 /, SC/987/
C
C     ON FIRST CALL, IF NO DATA UNCOMMENTED, TEST MACHINE TYPES.
      IF (SC .NE. 987) THEN
         DMACH(1) = 1.D13
         IF (      SMALL(1) .EQ. 1117925532
     *       .AND. SMALL(2) .EQ. -448790528) THEN
*           *** IEEE BIG ENDIAN ***
            SMALL(1) = 1048576
            SMALL(2) = 0
            LARGE(1) = 2146435071
            LARGE(2) = -1
            RIGHT(1) = 1017118720
            RIGHT(2) = 0
            DIVER(1) = 1018167296
            DIVER(2) = 0
            LOG10(1) = 1070810131
            LOG10(2) = 1352628735
         ELSE IF ( SMALL(2) .EQ. 1117925532
     *       .AND. SMALL(1) .EQ. -448790528) THEN
*           *** IEEE LITTLE ENDIAN ***
            SMALL(2) = 1048576
            SMALL(1) = 0
            LARGE(2) = 2146435071
            LARGE(1) = -1
            RIGHT(2) = 1017118720
            RIGHT(1) = 0
            DIVER(2) = 1018167296
            DIVER(1) = 0
            LOG10(2) = 1070810131
            LOG10(1) = 1352628735
         ELSE IF ( SMALL(1) .EQ. -2065213935
     *       .AND. SMALL(2) .EQ. 10752) THEN
*               *** VAX WITH D_FLOATING ***
            SMALL(1) = 128
            SMALL(2) = 0
            LARGE(1) = -32769
            LARGE(2) = -1
            RIGHT(1) = 9344
            RIGHT(2) = 0
            DIVER(1) = 9472
            DIVER(2) = 0
            LOG10(1) = 546979738
            LOG10(2) = -805796613
         ELSE IF ( SMALL(1) .EQ. 1267827943
     *       .AND. SMALL(2) .EQ. 704643072) THEN
*               *** IBM MAINFRAME ***
            SMALL(1) = 1048576
            SMALL(2) = 0
            LARGE(1) = 2147483647
            LARGE(2) = -1
            RIGHT(1) = 856686592
            RIGHT(2) = 0
            DIVER(1) = 873463808
            DIVER(2) = 0
            LOG10(1) = 1091781651
            LOG10(2) = 1352628735
         ELSE IF ( SMALL(1) .EQ. 1120022684
     *       .AND. SMALL(2) .EQ. -448790528) THEN
*           *** CONVEX C-1 ***
            SMALL(1) = 1048576
            SMALL(2) = 0
            LARGE(1) = 2147483647
            LARGE(2) = -1
            RIGHT(1) = 1019215872
            RIGHT(2) = 0
            DIVER(1) = 1020264448
            DIVER(2) = 0
            LOG10(1) = 1072907283
            LOG10(2) = 1352628735
         ELSE IF ( SMALL(1) .EQ. 815547074
     *       .AND. SMALL(2) .EQ. 58688) THEN
*           *** VAX G-FLOATING ***
            SMALL(1) = 16
            SMALL(2) = 0
            LARGE(1) = -32769
            LARGE(2) = -1
            RIGHT(1) = 15552
            RIGHT(2) = 0
            DIVER(1) = 15568
            DIVER(2) = 0
            LOG10(1) = 1142112243
            LOG10(2) = 2046775455
         ELSE
            DMACH(2) = 1.D27 + 1
            DMACH(3) = 1.D27
            LARGE(2) = LARGE(2) - RIGHT(2)
            IF (LARGE(2) .EQ. 64 .AND. SMALL(2) .EQ. 0) THEN
               CRAY1(1) = 67291416
               DO 10 J = 1, 20
                  CRAY1(J+1) = CRAY1(J) + CRAY1(J)
 10               CONTINUE
               CRAY1(22) = CRAY1(21) + 321322
               DO 20 J = 22, 37
                  CRAY1(J+1) = CRAY1(J) + CRAY1(J)
 20               CONTINUE
               IF (CRAY1(38) .EQ. SMALL(1)) THEN
*                  *** CRAY ***
                  CALL I1MCRY(SMALL(1), J, 8285, 8388608, 0)
                  SMALL(2) = 0
                  CALL I1MCRY(LARGE(1), J, 24574, 16777215, 16777215)
                  CALL I1MCRY(LARGE(2), J, 0, 16777215, 16777214)
                  CALL I1MCRY(RIGHT(1), J, 16291, 8388608, 0)
                  RIGHT(2) = 0
                  CALL I1MCRY(DIVER(1), J, 16292, 8388608, 0)
                  DIVER(2) = 0
                  CALL I1MCRY(LOG10(1), J, 16383, 10100890, 8715215)
                  CALL I1MCRY(LOG10(2), J, 0, 16226447, 9001388)
               ELSE
                  WRITE(*,9000)
                  STOP 779
                  END IF
            ELSE
               WRITE(*,9000)
               STOP 779
               END IF
            END IF
         SC = 987
         END IF
*    SANITY CHECK
      IF (DMACH(4) .GE. 1.0D0) STOP 778
      IF (I .LT. 1 .OR. I .GT. 5) THEN
         WRITE(*,*) 'D1MACH(I): I =',I,' is out of bounds.'
         STOP
         END IF
      D1MACH = DMACH(I)
      RETURN
 9000 FORMAT(/' Adjust D1MACH by uncommenting data statements'/
     *' appropriate for your machine.')
* /* Standard C source for D1MACH -- remove the * in column 1 */
*#include <stdio.h>
*#include <float.h>
*#include <math.h>
*double d1mach_(long *i)
*{
*	switch(*i){
*	  case 1: return DBL_MIN;
*	  case 2: return DBL_MAX;
*	  case 3: return DBL_EPSILON/FLT_RADIX;
*	  case 4: return DBL_EPSILON;
*	  case 5: return log10((double)FLT_RADIX);
*	  }
*	fprintf(stderr, "invalid argument: d1mach(%ld)\n", *i);
*	exit(1); return 0; /* some compilers demand return values */
*}
      END
      SUBROUTINE I1MCRY(A, A1, B, C, D)
**** SPECIAL COMPUTATION FOR OLD CRAY MACHINES ****
      INTEGER A, A1, B, C, D
      A1 = 16777216*B + C
      A = 16777216*A1 + D
      END
